/*
 *   IRC - Internet Relay Chat, modules/m_die.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

extern void sigterm_handler(int sig);

static struct Message _msgtab[] = {
    {MSG_DIE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_die, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.5 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
}
#else
void m_die_init(void)
{
    mod_add_cmd(_msgtab);
}
#endif

int m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;
    dlink_node *ptr;
    char *pass = ServerInfo.diepass;
    char passarr[PASSWDLEN];

    if (!OPCanDie(sptr)) {
	send_me_numeric(sptr, ERR_NOPRIVILEGES);
	return 0;
    }

    if (pass) {
	if (parc < 2) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_DIE);
	    return 0;
	}
	if (strcmp(pass, calcpass(parv[1], passarr))) {
	    send_me_numeric(sptr, ERR_PASSWDMISMATCH);
	    return 0;
	}
    }

    for (ptr = lclient_list.head; ptr; ptr = ptr->next) {
	if (!(acptr = ptr->data))
	    continue;
	if (IsClient(acptr))
	    send_me_notice(acptr, ":Server Terminating. %s", sptr->name);
	else if (IsServer(acptr))
	    sendto_one_server(acptr, &me, TOK1_ERROR,
			      ":Terminated by %s", get_client_name(sptr, TRUE));
    }
    logevent_call(LogSys.operevent, MSG_DIE, sptr, &parv, parc);
    sigterm_handler(0);
    return 0;
}
