/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_ison.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "msg.h"
#include "numeric.h"
#include "language.h"

static char buf[BUFSIZE];

static struct Message _msgtab[] = {
    {MSG_ISON, 0, 1, M_SLOW, 0L,
     m_unregistered, m_ison, m_ignore, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.3 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
}
#else
void m_ison_init(void)
{
    mod_add_cmd(_msgtab);
}
#endif

/*
 * m_ison added by Darren Reed 13/8/91 to act as an efficent user
 * indicator with respect to cpu/bandwidth used. Implemented for NOTIFY
 * feature in clients. Designed to reduce number of whois requests. Can
 * process nicknames in batches as long as the maximum buffer length.  
 * 
 * format: ISON :nicklist
 */

int m_ison(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;
    char *s, **pav = parv;
    char *p = (char *) NULL;
    int len;
    int len2;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_ISON);
	return 0;
    }

    (void) ircsprintf(buf, rpl_str(RPL_ISON), &me, *parv);
    len = strlen(buf);
    for (s = strtoken(&p, *++pav, " "); s; s = strtoken(&p, (char *) NULL, " "))
	if ((acptr = find_person(s))) {
	    len2 = strlen(acptr->name);
	    if ((len + len2 + 5) < sizeof(buf)) {
		(void) strcat(buf, acptr->name);
		len += len2;
		(void) strcat(buf, " ");
		len++;
	    } else
		break;
	}
    sendto_one(sptr, "%s", buf);
    return 0;
}
