/*
 *   IRC - Internet Relay Chat, modules/m_mode.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_mode.c,v 5.15 2003/04/01 15:07:43 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "msg.h"
#include "h.h"
#include "language.h"

static char modebuf[REALMODEBUFLEN], parabuf[REALMODEBUFLEN];

static char *m_token = TOK1_MODE;
static char *t_token = TOK1_TMODE;

static struct Message m_msgtab[] = {
    {MSG_MODE, 0, MAXPARA, M_SLOW | M_FLOOD_END, 0L,
     m_unregistered, m_mode, m_mode, m_mode, m_ignore}
};

static struct Message t_msgtab[] = {
    {MSG_TMODE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_tmode, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.15 $";

void _modinit(void)
{
    mod_add_cmd(m_msgtab);
    mod_add_cmd(t_msgtab);
    tok1_msgtab[(u_char) *m_token].msg = m_msgtab;
    tok1_msgtab[(u_char) *t_token].msg = t_msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(m_msgtab);
    mod_del_cmd(t_msgtab);
    tok1_msgtab[(u_char) *m_token].msg = NULL;
    tok1_msgtab[(u_char) *t_token].msg = NULL;
}
#else
void m_mode_init(void)
{
    mod_add_cmd(m_msgtab);
    mod_add_cmd(t_msgtab);
    tok1_msgtab[(u_char) *m_token].msg = m_msgtab;
    tok1_msgtab[(u_char) *t_token].msg = t_msgtab;
}
#endif

int m_mode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int mcount = 0;
    aChannel *chptr;

    if (parc > 1) {
	chptr = find_channel(parv[1]);
	if (!chptr)
	    return m_umode(cptr, sptr, parc, parv);
    } else {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	return 0;
    }

    if (!check_channelname(sptr, (unsigned char *) parv[1]))
	return 0;

    *modebuf = *parabuf = '\0';

    if (parc < 3) {
	modebuf[1] = '\0';
	channel_modes(sptr, modebuf, parabuf, chptr);
	send_me_numeric(sptr, RPL_CHANNELMODEIS, chptr, modebuf, parabuf);
	send_me_numeric(sptr, RPL_CREATIONTIME, chptr, chptr->tsval);
	return 0;
    }

    mcount = set_mode(cptr, sptr, chptr, parc - 2, parv + 2, modebuf, parabuf);

    if (strlen(modebuf) > (size_t) 1) {
	switch (mcount) {
	    case 0:
		break;
	    case -1:
		if (MyClient(sptr))
		    send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr);
		else
		    ircstp->is_fake++;
		break;
	    default:
		sendto_channel_butserv(chptr, sptr, TOK1_MODE, CHFL_GETSETTER,
					   "%s %s", modebuf, parabuf);
		sendto_match_servs(chptr, sptr, TOK1_MODE, "%s %s", modebuf, parabuf);
		sendto_service(SERVICE_SEE_MODES, 0, sptr, chptr, TOK1_MODE,
			       "%s %s", modebuf, parabuf);
	}
    }
    ClearOperMode(sptr);
    return 0;
}

int m_tmode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int mcount = 0;
    unsigned long tts = 0;
    aChannel *chptr;

    if (parc < 5)
	return 0;

    /* !ID TMODE #channel timestamp modes parameters */

    if (!(chptr = find_channel(parv[1])))
        return 0;

    *modebuf = *parabuf = '\0';

    tts = strtoul(parv[2], NULL, 0);
    if (tts > chptr->tsval) {
	sendto_lev(DEBUG_LEV, "Got TMODE from %C for %H with wrong timestamp. (%lu != %T)",
		   cptr, chptr, tts, chptr);
	return 0;
    }

    mcount = set_mode(cptr, sptr, chptr, parc - 3, parv + 3, modebuf, parabuf);

    if (strlen(modebuf) > (size_t) 1) {
        switch (mcount) {
            case 0:
                break;
            case -1:
                ircstp->is_fake++;
                break;
            default:
                sendto_channel_butserv(chptr, sptr, TOK1_MODE, CHFL_GETSETTER,
                                           "%s %s", modebuf, parabuf);
                sendto_service(SERVICE_SEE_MODES, 0, sptr, chptr, TOK1_MODE,
                               "%s %s", modebuf, parabuf);
                sendto_flag_serv_butone(NULL, CAPAB_TMODE, 0, sptr, TOK1_TMODE, "%H %T %s %s",
                                        chptr, chptr, modebuf, parabuf);
                sendto_flag_serv_butone(NULL, 0, CAPAB_TMODE, sptr, TOK1_MODE, "%H %s %s",
                                        chptr, modebuf, parabuf);
        }
    }
    return 0;
}

static char buf[BUFSIZE];

/*   
 * m_umode() added 15/10/91 By Darren Reed. parv[0] - sender parv[1] -
 * username to change mode for parv[2] - modes to change
 */
int m_umode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int flag;
    int *s;
    char **p, *m;
    aClient *acptr;
    int what;
    long setflags;
    int badflag = NO;
    dlink_node *ptr;

    what = CMODE_ADD;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_MODE);
	return 0;
    }

    if (!(acptr = find_person(parv[1]))) {
	if (MyConnect(sptr))
	    send_me_numeric(sptr, ERR_NOSUCHNICK, parv[1]);
	return 0;
    }

    if ((IsServer(sptr) || (sptr != acptr) || (acptr->from != sptr->from))) {
	if (!IsServer(cptr))
	    send_me_numeric(sptr, ERR_USERSDONTMATCH);
	return 0;
    }
    if (parc < 3) {
	m = buf;
	*m++ = '+';
	for (s = user_modes; (flag = *s) && (m - buf < BUFSIZE - 4); s += 2)
	    if (sptr->umode & flag)
		*m++ = (char) (*(s + 1));
	*m = '\0';
	send_me_numeric(sptr, RPL_UMODEIS, buf);
	return 0;
    }

    /*
     * find flags already set for user
     */
    setflags = 0;
    for (s = user_modes; (flag = *s); s += 2)
	if (sptr->umode & flag)
	    setflags |= flag;
    /*
     * parse mode change string(s)
     */
    for (p = &parv[2]; p && *p; p++)
	for (m = *p; *m; m++)
	    switch (*m) {
		case '+':
		    what = CMODE_ADD;
		    break;
		case '-':
		    what = CMODE_DEL;
		    break;
		    /*
		     * we may not get these, but they shouldnt be in
		     * default
		     */
		case ' ':
		case '\r':
		case '\n':
		case '\t':
		case 'h':
		case 'r':
		    break;	/* users can't set themselves +r or +h! */
		case 'L':
		    if (!MyClient(sptr)) {
			if ((parc > 4) && IsDigit(parv[3][0])) {
			    if (sptr->tsval == strtoul(parv[3], NULL, 0)) {
				if (IsDigit(parv[4][0])) 
			    	    sptr->lang = set_language(atoi(parv[4]));
			    }
			}
		    }
		    break;

		case 'A':
		    /*   
		     * set auto +a if user is setting +A
		     */
		    if (MyClient(sptr) && (what == CMODE_ADD))
			sptr->umode |= UMODE_a;
		default:
		    for (s = user_modes; (flag = *s); s += 2)
			if (*m == (char) (*(s + 1))) {
			    if (what == CMODE_ADD)
				sptr->umode |= flag;
			    else
				sptr->umode &= ~flag;
			    break;
			}
		    if (flag == 0 && MyConnect(sptr))
			badflag = YES;
		    break;
	    }

    if (badflag)
	send_me_numeric(sptr, ERR_UMODEUNKNOWNFLAG);

    if ((!(setflags & UMODE_o) && IsAnOper(sptr)) && MyConnect(sptr))
	ClearOper(sptr);

    if ((setflags & UMODE_o) && !IsAnOper(sptr) && MyConnect(sptr)) {
	det_confs_butmask(sptr, CONF_CLIENT & ~CONF_OPERATOR);
	sptr->sendqlen = get_sendq(sptr);
	sptr->oflag = 0;
        ptr = dlinkFind(&locoper_list, sptr);
        if (ptr != NULL) {
  	    dlinkDelete(ptr, &locoper_list);
            free_dlink_node(ptr);
        }
    }

    if (!(setflags & UMODE_o) && IsAnOper(sptr))
	Count.oper++;

    if ((setflags & UMODE_o) && !IsAnOper(sptr)) 
	Count.oper--;

    if (!(setflags & UMODE_i) && IsInvisible(sptr))
	Count.invisi++;
    if ((setflags & UMODE_i) && !IsInvisible(sptr))
	Count.invisi--;

    if (!IsAnOper(sptr)) {
	if (IsAdmin(sptr))
	    ClearAdmin(sptr);
	if (IsSAdmin(sptr))
	    ClearSAdmin(sptr);
	if (IsRouting(sptr))
	    ClearRouting(sptr);
    }
    if (MyClient(sptr)) {
	if (IsAdmin(sptr) && !OPIsAdmin(sptr))
	    ClearAdmin(sptr);
	if (IsSAdmin(sptr) && !OPIsSAdmin(sptr))
	    ClearSAdmin(sptr);
    }
    send_umode_out(cptr, sptr, setflags);

    return 0;
}
