/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_motd.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_motd.c,v 5.5 2002/06/18 10:09:25 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_MOTD;

static struct Message _msgtab[] = {
    {MSG_MOTD, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, u_motd, o_motd, u_motd, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.5 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_motd_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/*
 * * m_motd 
 *       parv[0] = sender prefix 
 *       parv[1] = servername
 */
int u_motd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    static time_t last_used = 0L;

    if (hunt_server(cptr, sptr, ":%s %s :%s", TOK1_MOTD, 1, parc, parv) != HUNTED_ISME)
	return 0;
    if ((last_used + ServerOpts.motd_wait) > NOW)
	return 0;
    else
	last_used = NOW;

    if (IsPerson(sptr))
	sendto_lev(SPY_LEV, "MOTD requested by %^C [%s]", sptr, sptr->user->server);
    send_message_file(&(GeneralOpts.motd), cptr, sptr, parc, parv);
    return 0;
}

int o_motd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    if (hunt_server(cptr, sptr, ":%s %s :%s", TOK1_MOTD, 1, parc, parv) != HUNTED_ISME)
	return 0;

    if (IsPerson(sptr))
	sendto_lev(SPY_LEV, "MOTD requested by %^C [%s]", sptr, sptr->user->server);
    send_message_file(&(GeneralOpts.motd), cptr, sptr, parc, parv);
    return 0;
}
