/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_myid.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_myid.c,v 5.4 2002/07/31 12:31:34 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_MYID;

static struct Message _msgtab[] = {
    {MSG_MYID, 0, MAXPARA, M_SLOW | M_IDLE_RESET, 0L,
     m_myid, m_ignore, m_ignore, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_myid_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/* m_myid
 * parv[1] = base64 server id
 */
int m_myid(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *base64id;

    if (cptr->id.string[0] || parc < 2 || !IsIDCapable(cptr) || cptr != sptr)
	return 0;

    base64id = parv[1];
    if (base64id[0] != '!' || base64id[1] == '\0' || !valid_base64_server_id(base64id + 1))
	return 0;
    else
	base64id++;

    strlcpy_irc(cptr->id.string, base64id, 8);
    return 0;
}
