/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_operdo.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "s_conf.h"
#include "h.h"

static char nickbuf[BUFSIZE], buf[BUFSIZE];
static char modebuf[REALMODEBUFLEN], parabuf[REALMODEBUFLEN];

static struct Message _msgtab[] = {
    {MSG_OPERDO, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_operdo, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.12 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
}
#else
void m_operdo_init(void)
{
    mod_add_cmd(_msgtab);
}
#endif

/*
 * m_operdo 
 * * -TimeMr14C
 * * operdo mode/kick/topic
 * * parv[0] = sender
 * * parv[1] = opcommand
 * * parv[2] = channel
 * * parv[3] = params
 */

static int get_opcommand(char *param)
{
    int i = 0;

    if (!irc_strcmp(param, "MODE")) {
	i = 1;
    } else if (!irc_strcmp(param, "TOPIC")) {
	i = 2;
    } else if (!irc_strcmp(param, "KICK")) {
	i = 3;
    } else if (!irc_strcmp(param, "JOIN")) {
	i = 4;
    } else if (!irc_strcmp(param, "HELP")) {
	i = 5;
    }
    return i;

}

int m_operdo(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

    aChannel *chptr = NULL;
    int o_param;
    int sendtsm = 0;
    time_t ts = 0;

    if (parc < 4)
	return 0;

    o_param = get_opcommand(parv[1]);

    if (o_param == 5)
	goto switching;

    if (!check_channelname(sptr, (unsigned char *) parv[2])) {
	send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[2]);
	return 0;
    }
    chptr = find_channel(parv[2]);
    if ((!chptr) && (o_param < 4)) {
	send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[2]);
	return 0;
    }
    if ((o_param < 4) && IsChanAnon(chptr)) {
	send_me_notice(sptr, ":OPERDO Cannot be used for anonymous channels");
	sendto_ops("from %C: %C tried to use OPERDO on anonymous channel %H", &me, sptr, chptr);
	return 0;
    }

    logevent_call(LogSys.operevent, MSG_OPERDO, sptr, &parv, parc);

    /* in OPERDO the command arrives from a non-chanop, therefore
     * there is no use in differentiating between ops and nonops
     * in a hideops case
     * -TimeMr14C
     */

  switching:
    switch (o_param) {

	case 1:{
		SetOperMode(sptr);
		sendtsm = set_mode(cptr, sptr, chptr, parc - 3, parv + 3, modebuf, parabuf);
		if (strlen(modebuf) > (size_t) 1) {
		    sendto_channel_butserv(chptr, sptr, TOK1_MODE, 0, "%s %s", modebuf, parabuf);
		    sendto_match_servs(chptr, cptr, TOK1_MODE, "%s %s", modebuf, parabuf);
		    sendto_serv_butone(NULL, &me, TOK1_GLOBOPS,
				       ":%C used OPERDO MODE on %H %s%s%s",
				       sptr, chptr, modebuf, (*parabuf != 0 ? " " : ""), parabuf);
		    sendto_ops
			("from %C: %C used OPERDO MODE on %H %s%s%s",
			 &me, sptr, chptr, modebuf, (*parabuf != 0 ? " " : ""), parabuf);
		}
		break;
	    }

	case 2:{
		ts = timeofday;
		strlcpy_irc(chptr->topic, parv[3], TOPICLEN);
		strcpy(chptr->topic_nick, sptr->name);
		chptr->topic_time = ts;

		sendto_match_servs(chptr, &me, TOK1_TOPIC, "%s %lu :%s",
				   chptr->topic_nick, chptr->topic_time, chptr->topic);
		sendto_channel_butserv(chptr, sptr, TOK1_TOPIC, 0, ":%s", chptr->topic);
		sendto_serv_butone(NULL, &me, TOK1_GLOBOPS,
				   ":%C used OPERDO TOPIC on %H", sptr, chptr);
		sendto_ops("from %C: %C used OPERDO TOPIC on %H", &me, sptr, chptr);
		break;
	    }
	case 3:{

    aClient *who;
    int chasing = 0;
    int user_count;
    char *comment, *name, *p = NULL, *user, *p2 = NULL;

		if (parc < 4 || *parv[2] == '\0') {
		    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_OPERDO);
		    return 0;
		}

		comment = (BadPtr(parv[4])) ? parv[0] : parv[4];

		if (strlen(comment) > (size_t) TOPICLEN)
		    comment[TOPICLEN] = '\0';

		*nickbuf = *buf = '\0';
		name = strtoken(&p, parv[2], ",");
		while (name) {
		    chptr = find_channel(name);
		    if (!chptr) {
			send_me_numeric(sptr, ERR_NOSUCHCHANNEL, name);
			name = strtoken(&p, (char *) NULL, ",");
			continue;
		    }

		    user = strtoken(&p2, parv[3], ",");
		    user_count = 4;
		    while (user && user_count) {
			user_count--;
			if (!(who = find_chasing(sptr, user, &chasing))) {
			    user = strtoken(&p2, (char *) NULL, ",");
			    continue;
			}

			if (IsMember(who, chptr)) {
			    sendto_channel_butserv(chptr, sptr, TOK1_KICK, 0,
						   "%s :%s", who->name, comment);
			    sendto_match_servs(chptr, cptr, TOK1_KICK, "%s :%s",
					       who->name, comment);
			    sendto_serv_butone(NULL, &me, TOK1_GLOBOPS,
					       ":%C used OPERDO KICK on %H for %C",
					       sptr, chptr, who);
			    sendto_ops
				("from %C: %C used OPERDO KICK on %C for %C",
				 &me, sptr, chptr, who);
			    remove_user_from_channel(who, chptr);
			} else
			    send_me_numeric(sptr, ERR_USERNOTINCHANNEL, user, name);
			user = strtoken(&p2, (char *) NULL, ",");
		    }
		    name = strtoken(&p, (char *) NULL, ",");
		}
		break;
	    }

	case 4:{
		privileged_join(sptr, cptr, parv[2]);
		chptr = find_channel(parv[2]);
		if (chptr == NULL)
		    break;
		sendtsm = set_mode(&me, &me, chptr, parc - 3, parv + 3, modebuf, parabuf);
		if (strlen(modebuf) > (size_t) 1) {

		    sendto_channel_butserv(chptr, sptr, TOK1_MODE, 0,
					   "%s %s", modebuf, parabuf);
		    sendto_match_servs(chptr, &me, TOK1_MODE, "%s %s", modebuf, parabuf);
		    sendto_serv_butone(NULL, &me, TOK1_GLOBOPS,
				       ":%C used OPERDO JOIN on %H %s%s%s",
				       sptr, chptr, modebuf, (*parabuf != 0 ? " " : ""), parabuf);
		    sendto_ops
			("from %C: %C used OPERDO JOIN on %H with %s%s%s",
			 &me, sptr, chptr, modebuf, (*parabuf != 0 ? " " : ""), parabuf);
		}
		break;
	    }

	case 5:{
		send_me_notice(sptr, ":Available commands are:");
		send_me_notice(sptr, ": ");
		send_me_notice(sptr, ":\002MODE\002 #channel params");
		send_me_notice(sptr, ":\002TOPIC\002 #channel :text");
		send_me_notice(sptr, ":\002KICK\002 #channel user :reason");
		send_me_notice(sptr, ":\002JOIN\002 #channel modes");
		break;
	    }

	case 0:{
		send_me_notice(sptr, ":No such operdo command %s", parv[1]);
		send_me_notice(sptr, ":Try \002HELP\002");
		break;
	    }
	default:
	    break;
    }
    return 0;

}
