/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_servlist.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"

static struct Message _msgtab[] = {
    {MSG_SERVLIST, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_servlist, m_servlist, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.3 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
}
#else
void m_servlist_init(void)
{
    mod_add_cmd(_msgtab);
}
#endif

int m_servlist(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

    aService *svc;
    aClient *acptr;
    char out[HOSTLEN];

    for (svc = firstservice; svc; svc = svc->next) {
	if ((acptr = svc->scptr) != NULL)
	    send_me_numeric(sptr, RPL_SERVLIST,
			    acptr->name, acptr->username,
			    (IsAnOper(sptr) ? acptr->sockhost : "<hidden>"),
			    (WillHideName(acptr) ? stealth_server(svc->server, out)
			     : svc->server), acptr->hopcount, acptr->info);
    }
    send_me_numeric(sptr, RPL_SERVLISTEND);
    return 2;
}
