/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_set.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "throttle.h"
#include "s_conf.h"
#include "h.h"

static struct Message s_msgtab[] = {
    {MSG_SET, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_set, m_ignore, m_ignore}
};
static struct Message ns_msgtab[] = {
    {MSG_NETSET, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, m_netset, m_netset, m_ignore}
};

static char *token1 = TOK1_NETSET;

static int log_set_debuglevel = -1;

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.11 $";

void _modinit(void)
{
    mod_add_cmd(s_msgtab);
    mod_add_cmd(ns_msgtab);
    tok1_msgtab[(u_char) *token1].msg = ns_msgtab;
    log_set_debuglevel = logevent_register("set debuglevel", LOG_ALWAYS, LOG_LOGFILE | LOG_IRCLOG, 
					   LOG_NOTICE, "\nIRCLOG: New DEBUGLEVEL %d at %s\n");
}

void _moddeinit(void)
{
    mod_del_cmd(s_msgtab);
    mod_del_cmd(ns_msgtab);
    tok1_msgtab[(u_char) *token1].msg = NULL;
}
#else
void m_set_init(void)
{
    mod_add_cmd(s_msgtab);
    mod_add_cmd(ns_msgtab);
    tok1_msgtab[(u_char) *token1].msg = ns_msgtab;
    log_set_debuglevel = logevent_register("set debuglevel", LOG_ALWAYS, LOG_LOGFILE | LOG_IRCLOG, 
					   LOG_NOTICE, "\nIRCLOG: New DEBUGLEVEL %d at %s\n");
}
#endif

int m_set(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *command;

    if (parc > 1) {
	command = parv[1];
	logevent_call(LogSys.operevent, MSG_SET, sptr, &parv, parc);
	if (!irc_strncmp(command, "MAX", 3)) {
	    if (parc > 2) {
    int new_value = atoi(parv[2]);

		if (new_value > MASTER_MAX) {
		    send_me_notice(sptr,
				   ":You cannot set MAXCLIENTS to > MASTER_MAX (%d)", MASTER_MAX);
		    return 0;
		}
		if (new_value < 32) {
		    send_me_notice(sptr,
				   ":You cannot set MAXCLIENTS to < 32 (%d)",
				   GeneralOpts.maxclients);
		    return 0;
		}
		GeneralOpts.maxclients = new_value;
		send_me_notice(sptr,
			       ":NEW MAXCLIENTS = %d (Current = %d)", GeneralOpts.maxclients, Count.local);
		sendto_ops("%C set new MAXCLIENTS to %d (%d current)", sptr,
			   GeneralOpts.maxclients, Count.local);
		return 0;
	    }
	    send_me_notice(sptr, ":Current Maxclients = %d (%d)", GeneralOpts.maxclients, Count.local);
	    return 0;
	}
	else if (!irc_strncmp(command, "SPAMNUM", 7)) {
	    if (parc > 2) {
    int newval = atoi(parv[2]);

		if (newval <= 0) {
		    send_me_notice(sptr, ":spam NUM must be > 0");
		    return 0;
		}
		if (newval < MIN_SPAM_NUM)
		    GeneralOpts.spam_num = MIN_SPAM_NUM;
		else
		    GeneralOpts.spam_num = newval;
		sendto_ops("%s has changed spam NUM to %i", parv[0], GeneralOpts.spam_num);
		send_me_notice(sptr, ":spam NUM is now set to %i", GeneralOpts.spam_num);
		return 0;
	    } else {
		send_me_notice(sptr, ":spam NUM is currently %i", GeneralOpts.spam_num);
		return 0;
	    }
	} else if (!irc_strncmp(command, "SPAMTIME", 8)) {
	    if (parc > 2) {
    int newval = atoi(parv[2]);

		if (newval <= 0) {
		    send_me_notice(sptr, ":spam TIME must be > 0");
		    return 0;
		}
		if (newval < MIN_SPAM_TIME)
		    GeneralOpts.spam_time = MIN_SPAM_TIME;
		else
		    GeneralOpts.spam_time = newval;
		sendto_ops("%s has changed spam TIME to %i", parv[0], GeneralOpts.spam_time);
		send_me_notice(sptr, ":SPAM TIME is now set to %i", GeneralOpts.spam_time);
		return 0;
	    } else {
		send_me_notice(sptr, ":spam TIME is currently %i", GeneralOpts.spam_time);
		return 0;
	    }
        } else if (!irc_strncmp(command, "LOGGING", 3)) {
            if (parc > 2) {

                if (!irc_strncmp(parv[2], "ON", 2)) {
                    send_me_notice(sptr,
                                   ":The ircd stats logging");
                    sendto_ops("%C activated LOGGING", sptr);
                    GeneralOpts.enable_logging = 1;
                } else if (!irc_strncmp(parv[2], "OFF", 3)) {
                    send_me_notice(sptr,
                                   ":The ircd stops logging");
                    sendto_ops("%C deactivated LOGGING", sptr);
                    GeneralOpts.enable_logging = 0;
                } else {
                    send_me_notice(sptr,
                                   ":The correct setting is either" "\002ON\002 or \002OFF\002");
                }

                return 0;
            } else {

                send_me_notice(sptr, ":LOGGING is now %s",
                               (GeneralOpts.enable_logging ? "\002ACTIVE\002" : "\002INACTIVE\002"));
                return 0;
            }
	} else if (!irc_strncmp(command, "DEBUG", 3)) {
	    if (parc > 2 && IsAdmin(sptr)) {
		if (atoi(parv[2])) {
		    GeneralOpts.debuglevel = atoi(parv[2]);
		    send_me_notice(sptr, ":DEBUGMODE set to level \002%d\002", GeneralOpts.debuglevel);
		    logevent_call(log_set_debuglevel, 
			   GeneralOpts.debuglevel, myctime(irc_time(NULL)));

		} else if (!irc_strncmp(parv[2], "OFF", 3)) {
		    logevent_call(log_set_debuglevel, -1, myctime(irc_time(NULL)));
		    GeneralOpts.debuglevel = 0;
		    send_me_notice(sptr, ":DEBUGMODE is now \002OFF\002");
		} else {
		    send_me_notice(sptr,
				   ":Possible settings are "
				   "\0021\002 to \0029\002 or \002OFF\002");
		}
		return 0;
	    } else {
		send_me_notice(sptr,
			       ":DEBUGMODE is set to \002%d\002",
			       GeneralOpts.debuglevel ? GeneralOpts.debuglevel : 0);
		return 0;
	    }
	}
        else if (!irc_strncmp(command, "THROTTLE", 8)) {
    char *changed = NULL;
    char *to = NULL;
            /* several values available:
             * ENABLE [on|off] to enable the code
             * COUNT [n] to set a max count, must be > 1
             * TIME [n] to set a max time before expiry, must be > 5
             * BANTIME [n] to set a time for the throttle bans to expire
             * HASH [n] to set the size of the hash table, must be bigger than
             *          the default */

            /* only handle individual settings if parc > 3 (they're actually 
             * changing stuff) */
            if (parc > 3) {
                if (!irc_strcmp(parv[2], "COUNT")) {
    int cnt;
                    changed = "COUNT";
                    cnt = atoi(parv[3]);
                    if (cnt > 1) {
                        throttle_tcount = cnt;   
                        to = parv[3];
                    }
                } else if (!irc_strcmp(parv[2], "TIME")) {
    int cnt;
                    changed = "TIME";
                    cnt = atoi(parv[3]);
                    if (cnt >= 5) {
                        throttle_ttime = cnt;    
                        to = parv[3];
                    }
                } else if (!irc_strcmp(parv[2], "BANTIME")) {
    int cnt;
                    changed = "BANTIME";
                    cnt = atoi(parv[3]);
                    if (cnt >= 30) {
                        throttle_ztime = cnt;       
                        to = parv[3];
                    }
                } else if (!irc_strcmp(parv[2], "HASH")) {
    int cnt;               
                    changed = "HASH";
                    cnt = atoi(parv[3]);
                    if (cnt >= THROTTLE_HASHSIZE) {    
                        throttle_resize(cnt);               
                        to = parv[3];            
                    }    
                }          

                if (to != NULL) {
                    sendto_ops("%s has changed throttle %s to %s", parv[0],
                               changed, to);
                    sendto_one(sptr, ":%s NOTICE %s :set throttle %s to %s",
                               me.name, parv[0], changed, to);
                }
            } else {
                /* report various things, we cannot easily get the hash size, so
                 * leave that alone. */
                send_me_notice(sptr, ":THROTTLE COUNT=%d", throttle_tcount);
                send_me_notice(sptr, ":THROTTLE TIME=%d", throttle_ttime);
                send_me_notice(sptr, ":THROTTLE BANTIME=%d", throttle_ztime);
            }
        }
    } else {
	send_me_notice(sptr, ":Options: MAX, LOGGING");
	send_me_notice(sptr, ":Options: SPAMNUM, SPAMTIME");
	send_me_notice(sptr, ":Options: DEBUG");
	send_me_notice(sptr, ":Options: THROTTLE <COUNT|TIME|BANTIME|HASH> [setting]");
    }
    return 0;
}

int m_netset(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    if (parc < 2)
	return 0;

    (void) m_set(cptr, sptr, parc, parv);

    if (parc == 3)
	sendto_serv_butone(cptr, sptr, TOK1_NETSET, "%s :%s", parv[1], parv[2]);
    else
	sendto_serv_butone(cptr, sptr, TOK1_NETSET, "%s", parv[1]);

    return 0;
}
