/************************************************************************
 * IRC - Internet Relay Chat, modules/m_setlang.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "numeric.h"
#include "sys.h"
#include "msg.h"
#include "h.h"
#include "confitem.h"
#include "language.h"

static struct Message _msgtab[] = {
    {MSG_SETLANG, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_setlang, m_setlang, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.8 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
}
#else
void m_setlang_init(void)
{
    mod_add_cmd(_msgtab);
}
#endif

int m_setlang(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_SETLANG);
	return 0;

    }

    if (irc_strcmp(parv[1], "list") == 0) {
	list_languages(sptr);
	return 0;
    }
    sptr->lang = lang_parse(parv[1]);
    if (sptr->lang) {
	send_me_notice(sptr, ":Language is %s", parv[1]);
	sendto_serv_butone(NULL, sptr, TOK1_MODE, "%~C +L %T %d", sptr, sptr, sptr->lang);
    } else
	send_me_notice(sptr, ":No such language");

    return 0;
}

