/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_sqline.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_sqline.c,v 5.6 2003/02/27 12:49:29 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_SQLINE;

static struct Message _msgtab[] = {
    {MSG_SQLINE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_sqline, m_sqline}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.6 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_sqline_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

int m_sqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aConfItem *aconf;

    if (!(IsServer(sptr) || IsULine(sptr)))
	return 0;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_SQLINE);
	return 0;
    }

    /*
     * get rid of redundancies 
     */
    parv[1] = collapse(parv[1]);

    /*
     * if we have any Q:lines (SQ or Q) that match
     * * this Q:line, just return (no need to waste cpu 
     */

    if ((aconf = find_conf_by_name(parv[1], CONF_QUARANTINE)) == NULL) {

	aconf = make_conf();

	/*
	 * Q:line and SQline, woo 
	 */
	aconf->status = (CONF_QUARANTINE | CONF_SQLINE);
	DupString(aconf->name, parv[1]);
	DupString(aconf->passwd, (parv[2] != NULL ? parv[2] : "Reserved"));
	aconf->next = GlobalConfItemList;
	GlobalConfItemList = aconf;
    }
    sendto_serv_butone(cptr, sptr, TOK1_SQLINE, "%s :%s", parv[1], aconf->passwd);
    return 0;
}
