/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_squery.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"

static char *token = TOK1_SQUERY;

static struct Message _msgtab[] = {
    {MSG_SQUERY, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_squery, m_squery, m_squery, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.5 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_squery_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/*
 * Send query to service.
 * parv[1] - string to match name against
 * parv[2] - string to send to service
 */

int m_squery(aClient *cptr, aClient *sptr, int parc, char *parv[])
{

    aClient *acptr;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NORECIPIENT, MSG_SQUERY);
	return 1;
    }
    if (parc == 2) {
	send_me_numeric(sptr, ERR_NOTEXTTOSEND);
	return 2;
    }

    if (((acptr = find_client(parv[1])) != NULL)
	&& (acptr->service)) {
	if (acptr->service->enable & SERVICE_SEE_PREFIX)
	    if (MyConnect(acptr))
		sendto_one(acptr, ":%C %s %s :%s", sptr, MSG_SQUERY, acptr->name, parv[2]);
	    else
		sendto_one_server(acptr, sptr, TOK1_SQUERY, "%~C :%s", acptr, parv[2]);
	else if (MyConnect(acptr))
	    sendto_one(acptr, ":%C %s %s :%s", sptr, MSG_SQUERY, acptr->name, parv[2]);
	else
	    sendto_one_server(acptr, sptr, TOK1_SQUERY, "%~C :%s", acptr, parv[2]);
    } else {
	send_me_numeric(sptr, ERR_NOSUCHNICK, parv[1]);
    }
    return 0;
}
