/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_Svsnoop.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_svsnoop.c,v 5.3 2002/06/02 14:22:26 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_SVSNOOP;

static struct Message _msgtab[] = {
    {MSG_SVSNOOP, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_svsnoop, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.3 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_svsnoop_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

/*
 * m_svsnoop - Remove all ops from a server
 * *  Once again pretty much straight outt df
 * *    - Raistlin
 * * parv[0] = sender prefix
 * * parv[1] = server 
 * * parv[2] = +/-
 */

int m_svsnoop(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aConfItem *aconf;

    if (!(IsULine(sptr) && parc > 2))
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s :%s", TOK1_SVSNOOP, 1, parc, parv) == HUNTED_ISME) {
	if (parv[2][0] == '+')
	    for (aconf = GlobalConfItemList; aconf; aconf = aconf->next) {
		if (aconf->status & CONF_OPERATOR)
		    aconf->status = CONF_ILLEGAL;
	} else
	    rehash(&me, &me, 2);
    }
    return 0;
}
