/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_szline.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_szline.c,v 5.4 2002/09/22 09:36:14 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_SZLINE;

static struct Message _msgtab[] = {
    {MSG_SZLINE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_szline, m_szline}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_szline_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

int m_szline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aConfItem *aconf;

    if (!(IsServer(sptr) || IsULine(sptr)))
	return 0;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_SZLINE);
	return 0;
    }

    if (!(aconf = find_is_zlined(parv[1]))) {
	/*
	 * okay, it doesn't suck, build a new conf for it 
	 */
	aconf = make_conf();
	aconf->status = CONF_SZLINE;
	DupString(aconf->host, parv[1]);
	DupString(aconf->passwd, (parv[2] != NULL ? parv[2] : "No Reason"));
	aconf->name = (char *) NULL;
	add_szline(aconf);
    }
    sendto_serv_butone(cptr, sptr, TOK1_SZLINE, "%s :%s", parv[1], aconf->passwd);
    return 0;
}
