/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_unjupiter.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_UNJUPE;

static struct Message _msgtab[] = {
    {MSG_UNJUPE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_permission, o_unjupiter, m_unjupiter, m_unjupiter}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.8 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_unjupiter_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

int o_unjupiter(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    if (ServerOpts.no_oper_jupiter)
        return m_permission(cptr, sptr, parc, parv);
    else
        return m_unjupiter(cptr, sptr, parc, parv);
}

int m_unjupiter(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int matchit = 0;
    char *mask;
    aConfItem *aconf, *ac2, *ac3;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_UNJUPE);
	return 0;
    }

    if (parc == 3) {
	matchit = atoi(parv[1]);
	mask = parv[2];
    } else
	mask = parv[1];

    ac2 = ac3 = aconf = GlobalConfItemList;
    while (aconf) {
	if ((aconf->status & CONF_JUPECHAN) &&
	    aconf->name && (matchit ? !match(mask, aconf->name)
			    : !irc_strcmp(mask, aconf->name))) {
	    if (GlobalConfItemList == aconf)
		ac2 = ac3 = GlobalConfItemList = aconf->next;
	    else
		ac2 = ac3->next = aconf->next;
	    MyFree(aconf->passwd);
	    MyFree(aconf->name);
	    MyFree(aconf);
	    aconf = ac2;
	} else {
	    ac3 = aconf;
	    aconf = aconf->next;
	}
    }
    logevent_call(LogSys.operevent, MSG_UNJUPE, sptr, &parv, parc);
    if (MyClient(sptr))
	sendto_gnotice
	    ("from %C: %s has UNJUPED following channel mask: %s", &me, sptr->name, parv[1], mask);

    if (parc == 3)
	sendto_serv_butone(cptr, sptr, TOK1_UNJUPE, "%d :%s", matchit, mask);
    else
	sendto_serv_butone(cptr, sptr, TOK1_UNJUPE, "%s", mask);
    return 0;
}
