/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_unsgline.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: m_unsgline.c,v 5.4 2002/06/02 14:22:26 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"

static char *token = TOK1_UNSGLINE;

static struct Message _msgtab[] = {
    {MSG_UNSGLINE, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_unsgline, m_unsgline},
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
}
#else
void m_unsgline_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
}
#endif

int m_unsgline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int matchit = 0;
    char *mask;
    aConfItem *aconf, *ac2, *ac3;

    if (!(IsServer(sptr) || IsULine(sptr)))
	return 0;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_UNSGLINE);
	return 0;
    }

    if (parc == 3) {
	matchit = atoi(parv[1]);
	mask = parv[2];
    } else
	mask = parv[1];

    /*
     * find the sgline and remove it 
     */
    /*
     * this was way too complicated and ugly. Fixed. -lucas 
     */
    /*
     * Changed this to use match.  Seems to make more sense? 
     */
    /*
     * Therefore unsgline * will clear the sgline list. 
     */
    ac2 = ac3 = aconf = GlobalConfItemList;
    while (aconf) {
	if ((aconf->status & (CONF_GCOS))
	    && (aconf->status & (CONF_SGLINE)) &&
	    aconf->name && (matchit ? !match(mask, aconf->name)
			    : !irc_strcmp(mask, aconf->name))) {
	    if (GlobalConfItemList == aconf)
		ac2 = ac3 = GlobalConfItemList = aconf->next;
	    else
		ac2 = ac3->next = aconf->next;
	    MyFree(aconf->passwd);
	    MyFree(aconf->name);
	    MyFree(aconf);
	    aconf = ac2;
	} else {
	    ac3 = aconf;
	    aconf = aconf->next;
	}
    }

    if (parc == 3)
	sendto_serv_butone(cptr, sptr, TOK1_UNSGLINE, "%d :%s", matchit, mask);
    else
	sendto_serv_butone(cptr, sptr, TOK1_UNSGLINE, ":%s", mask);
    return 0;
}
