/*
 *   IRC - Internet Relay Chat, server/s_kline.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: s_kline.c,v 5.6 2002/09/22 09:36:15 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "s_conf.h"
#include <sys/stat.h>
#include <fcntl.h>

int flush_write(aClient *sptr, char *opernick, int out, 
		char *buf, int ntowrite, char *temppath);


/*
 * isnumber()
 *
 * inputs
 * - pointer to ascii string in output
 * - 0 if not an integer number, else the number side effects
 * - none return -1 if not an integer.
 * (if someone types in maxint, oh well..) - lucas
 */

int k_isnumber(char *p)
{
    int result = 0;
    while (*p) {
	if (IsDigit(*p)) {
	    result *= 10;
	    result += ((*p) & 0xF);
	    p++;
	} else
	    return (-1);
    }
    /*
     * in the degenerate case where oper does a /quote kline 0 user@host
     * :reason i.e. they specifically use 0, I am going to return 1
     * instead as a return value of non-zero is used to flag it as a
     * temporary kline
     */

    /*
     * er, no. we return 0 because 0 means that it's a permanent kline. -lucas
     * oh, and we only do this if DEFAULT_KLINE_TIME is specified.
     */

    if (!ServerOpts.default_kline_time) {
	if (result == 0)
	    result = 1;
    }
    return (result);
}

/*
 * report_matching_host_klines
 *
 * inputs               - aClient pointer pointing to user who requested stats             
 * k output             - NONE side effects     -
 *
 * report klines that are in the same "close" domain as user
 *
 * -Dianora
 */

void report_matching_host_klines(aClient *cptr, char *host)
{
    char *pass;
    char *name = (char *) NULL;
    char *found_host = (char *) NULL;
    aConfItem *tmp;
    aConfList *list;
    static char null[] = "<NULL>";
    char rev[HOSTLEN + 1];

    if (strlen(host) > (size_t) HOSTLEN)
	return;

    reverse(rev, host);

    /*               
     * Start with hostnames of the form "*word" (most frequent) -Sol
     */

    list = &KList2;
    while ((tmp = find_matching_conf(list, rev)) != NULL) {
	pass = BadPtr(tmp->passwd) ? null : tmp->passwd;
	name = BadPtr(tmp->name) ? null : tmp->name;
	found_host = BadPtr(tmp->host) ? null : tmp->host;
	if (tmp->status == CONF_KILL)
	    send_me_numeric(cptr, RPL_STATSKLINE, 'K', found_host, name, 0, pass);
	list = NULL;
    }

    /*
     * Try hostnames of the form "word*" -Sol
     */

    list = &KList1;
    while ((tmp = find_matching_conf(list, host)) != NULL) {
	pass = BadPtr(tmp->passwd) ? null : tmp->passwd;
	name = BadPtr(tmp->name) ? null : tmp->name;
	found_host = BadPtr(tmp->host) ? null : tmp->host;
	if (tmp->status == CONF_KILL)
	    send_me_numeric(cptr, RPL_STATSKLINE, 'K', found_host, name, 0, pass);
	list = NULL;
    }

    /*
     * If none of the above worked, try non-sorted entries -Sol
     */

    list = &KList3;
    while ((tmp = l_find_matching_conf(list, host)) != NULL) {
	pass = BadPtr(tmp->passwd) ? null : tmp->passwd;
	name = BadPtr(tmp->name) ? null : tmp->name;
	found_host = BadPtr(tmp->host) ? null : tmp->host;
	if (tmp->status == CONF_KILL)
	    send_me_numeric(cptr, RPL_STATSKLINE, 'K', found_host, name, 0, pass);
	list = NULL;
    }
}

/*
 * add_temp_kline
 *
 * inputs               - pointer to aConfItem output           - none Side
 * effects      - links in given aConfItem into temporary kline link
 * list
 *  
 * -Dianora
 */

void add_temp_kline(aConfItem *aconf)
{
    aconf->next = temporary_klines;
    temporary_klines = aconf;
}

/*
 * flush_temp_klines
 * 
 * inputs               - NONE output           - NONE side effects     - All
 * temporary klines are flushed out. really should be used only for
 * cases of extreme goof up for now.
 */
void flush_temp_klines()
{
aConfItem *kill_list_ptr;

    if ((kill_list_ptr = temporary_klines)) {
	while (kill_list_ptr) {
	    temporary_klines = kill_list_ptr->next;
	    MyFree(kill_list_ptr->host);
	    MyFree(kill_list_ptr->name);
	    MyFree(kill_list_ptr->passwd);
	    MyFree(kill_list_ptr);
	    kill_list_ptr = temporary_klines;
	}
    }
}

/*
 * report_temp_klines
 *
 * inputs               - aClient pointer output                - NONE side
 * effects      -
 *
 */
void report_temp_klines(aClient *sptr, int conftype, char letter)
{
    aConfItem *kill_list_ptr;
    aConfItem *last_list_ptr;
    aConfItem *tmp_list_ptr;
    char *host, *name, *reason;
    int len;

    if (temporary_klines) {
	kill_list_ptr = last_list_ptr = temporary_klines;

	while (kill_list_ptr) {
	    if (kill_list_ptr->hold <= NOW && kill_list_ptr->hold != 0xFFFFFFFF) {
		/*
		 * kline has expired
		 */
		if (temporary_klines == kill_list_ptr) {
		    temporary_klines = last_list_ptr = tmp_list_ptr = kill_list_ptr->next;
		} else {
		    tmp_list_ptr = last_list_ptr->next = kill_list_ptr->next;
		}

		MyFree(kill_list_ptr->host);
		MyFree(kill_list_ptr->name);
		MyFree(kill_list_ptr->passwd);
		MyFree(kill_list_ptr);
		kill_list_ptr = tmp_list_ptr;
	    } else {
		if (kill_list_ptr->host)
		    host = kill_list_ptr->host;
		else
		    host = "*";

		if (kill_list_ptr->name)
		    name = kill_list_ptr->name;
		else
		    name = "*";

		if (kill_list_ptr->passwd)
		    reason = kill_list_ptr->passwd;
		else
		    reason = "No Reason";

                if (kill_list_ptr->hold == 0xFFFFFFFF) {
                    len = 0;
                } else {
                    len = (kill_list_ptr->hold - NOW) / 60;
                }

		if (kill_list_ptr->status == conftype) {
		    send_me_numeric(sptr, RPL_STATSKLINE, letter, host, name, len, reason);
		}

		last_list_ptr = kill_list_ptr;
		kill_list_ptr = kill_list_ptr->next;
	    }
	}
    }
}

/*
 * third param (type):
 * 0: remove anything matching
 * CONF_AKILL: remove only CONF_AKILL
 * CONF_KILL: remove only CONF_KILL
 * or whatever.   - lucas
 * also returns number removed. - lucas
 */

int remove_temp_kline(char *host, char *user, int type)
{
    aConfItem *kill_list_ptr;
    aConfItem *last_list_ptr;
    aConfItem *tmp_list_ptr;
    int num_removed = 0;

    if (temporary_klines) {
	kill_list_ptr = last_list_ptr = temporary_klines;

	while (kill_list_ptr) {
	    if (!irc_strcmp(kill_list_ptr->host, host) &&
		!irc_strcmp(kill_list_ptr->name, user) && (!type || type == kill_list_ptr->status)) {
		if (temporary_klines == kill_list_ptr)
		    temporary_klines = last_list_ptr = tmp_list_ptr = kill_list_ptr->next;
		else
		    tmp_list_ptr = last_list_ptr->next = kill_list_ptr->next;

		MyFree(kill_list_ptr->host);
		MyFree(kill_list_ptr->name);
		MyFree(kill_list_ptr->passwd);
		MyFree(kill_list_ptr);
		kill_list_ptr = tmp_list_ptr;
		num_removed++;
	    } else {
		last_list_ptr = kill_list_ptr;
		kill_list_ptr = kill_list_ptr->next;
	    }
	}
    }
    return num_removed;
}

void do_rehash_akills(void)
{
    aConfItem *kill_list_ptr;
    aConfItem *last_list_ptr;
    aConfItem *tmp_list_ptr;

    if (temporary_klines) {
	kill_list_ptr = last_list_ptr = temporary_klines;

	while (kill_list_ptr) {
	    if (kill_list_ptr->hold <= NOW || kill_list_ptr->hold == 0xFFFFFFFF) {
		if (temporary_klines == kill_list_ptr)
		    temporary_klines = last_list_ptr = tmp_list_ptr = kill_list_ptr->next;
		else
		    tmp_list_ptr = last_list_ptr->next = kill_list_ptr->next;

		MyFree(kill_list_ptr->host);
		MyFree(kill_list_ptr->name);
		MyFree(kill_list_ptr->passwd);
		MyFree(kill_list_ptr);
		kill_list_ptr = tmp_list_ptr;
	    } else {
		last_list_ptr = kill_list_ptr;
		kill_list_ptr = kill_list_ptr->next;
	    }
	}
    }
}

extern aConfItem *temporary_excludes;

void do_rehash_excludes()
{
aConfItem *exclude_list_ptr;
aConfItem *last_list_ptr;
aConfItem *tmp_list_ptr;

    if (temporary_excludes) {
	exclude_list_ptr = last_list_ptr = temporary_excludes;

	while (exclude_list_ptr) {
	    if (exclude_list_ptr->hold <= NOW || exclude_list_ptr->hold == 0xFFFFFFFF) {
		if (temporary_excludes == exclude_list_ptr)
		    temporary_excludes = last_list_ptr = tmp_list_ptr = exclude_list_ptr->next;
		else
		    tmp_list_ptr = last_list_ptr->next = exclude_list_ptr->next;

		MyFree(exclude_list_ptr->host);
		MyFree(exclude_list_ptr->name);
		MyFree(exclude_list_ptr->passwd);
		MyFree(exclude_list_ptr);
		exclude_list_ptr = tmp_list_ptr;
	    } else {
		last_list_ptr = exclude_list_ptr;
		exclude_list_ptr = exclude_list_ptr->next;
	    }
	}
    }
}

void add_temp_exclude(aConfItem *aconf)
{
    aconf->next = temporary_excludes;
    temporary_excludes = aconf;
}

void report_temp_excludes(aClient *sptr)
{
    aConfItem *exclude_list_ptr;
    aConfItem *last_list_ptr;
    aConfItem *tmp_list_ptr;
    char *host, *name, *reason, type;
    int len;

    if (temporary_excludes) {
	exclude_list_ptr = last_list_ptr = temporary_excludes;

	while (exclude_list_ptr) {
	    if (exclude_list_ptr->hold <= NOW && exclude_list_ptr->hold != 0xFFFFFFFF) {
		if (temporary_excludes == exclude_list_ptr) {
		    temporary_excludes = last_list_ptr = tmp_list_ptr = exclude_list_ptr->next;
		} else {
		    tmp_list_ptr = last_list_ptr->next = exclude_list_ptr->next;
		}

		MyFree(exclude_list_ptr->host);
		MyFree(exclude_list_ptr->name);
		MyFree(exclude_list_ptr->passwd);
		MyFree(exclude_list_ptr);
		exclude_list_ptr = tmp_list_ptr;
	    } else {
		if (exclude_list_ptr->host)
		    host = exclude_list_ptr->host;
		else
		    host = "*";

		if (exclude_list_ptr->name)
		    name = exclude_list_ptr->name;
		else
		    name = "*";
		if (exclude_list_ptr->passwd)
		    reason = exclude_list_ptr->passwd;
		else
		    reason = "No Reason";

		if (exclude_list_ptr->hold == 0xFFFFFFFF) {
		    len = 0;
		} else {
		    len = (exclude_list_ptr->hold - NOW) / 60;
		}

		type = 'E';
		send_me_numeric(sptr, RPL_STATSKLINE, type, host, name, len, reason);

		last_list_ptr = exclude_list_ptr;
		exclude_list_ptr = exclude_list_ptr->next;
	    }
	}
    }
}

int remove_temp_exclude(char *host, char *user)
{
    aConfItem *exclude_list_ptr;
    aConfItem *last_list_ptr;
    aConfItem *tmp_list_ptr;
    int num_removed = 0;

    if (temporary_excludes) {
	exclude_list_ptr = last_list_ptr = temporary_excludes;

	while (exclude_list_ptr) {
	    if (!irc_strcmp(exclude_list_ptr->host, host)
		&& !irc_strcmp(exclude_list_ptr->name, user)) {
		if (temporary_excludes == exclude_list_ptr)
		    temporary_excludes = last_list_ptr = tmp_list_ptr = exclude_list_ptr->next;
		else
		    tmp_list_ptr = last_list_ptr->next = exclude_list_ptr->next;

		MyFree(exclude_list_ptr->host);
		MyFree(exclude_list_ptr->name);
		MyFree(exclude_list_ptr->passwd);
		MyFree(exclude_list_ptr);
		exclude_list_ptr = tmp_list_ptr;
		num_removed++;
	    } else {
		last_list_ptr = exclude_list_ptr;
		exclude_list_ptr = exclude_list_ptr->next;
	    }
	}
    }
    return num_removed;
}

void add_szline(aConfItem *aconf)
{
    aconf->next = szlines;
    szlines = aconf;
}

void remove_szline(char *mask, int m)
{
    aConfItem *aconf, *ac2, *ac3;

    ac2 = ac3 = aconf = szlines;
    while (aconf) {
	if ((aconf->status & (CONF_SZLINE)) && aconf->host && 
	    (m == 1 ? !match(mask, aconf->host) : !irc_strcmp(mask, aconf->host))) {
	    if (aconf == szlines)
		ac2 = ac3 = szlines = aconf->next;
	    else
		ac2 = ac3->next = aconf->next;
	    MyFree(aconf->passwd);
	    MyFree(aconf->host);
	    MyFree(aconf);
	    aconf = ac2;
	} else {
	    ac3 = aconf;
	    aconf = aconf->next;
	}
    }
}

void report_szlines(aClient *sptr)
{
    aConfItem *tmp;

    tmp = szlines;
    while (tmp) {
	send_me_numeric(sptr, RPL_STATSZLINE, 'Z', tmp->host,
			BadPtr(tmp->passwd) ? "No Reason" : tmp->passwd);
	tmp = tmp->next;
    }
}

void send_szlines(aClient *cptr)
{
    aConfItem *tmp;

    tmp = szlines;
    while (tmp) {
	sendto_one_server(cptr, &me, TOK1_SZLINE, "%s :%s", tmp->host, tmp->passwd);
	tmp = tmp->next;
    }
}
