/*
 *   IRC - Internet Relay Chat, chanmodes/chanmodes.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: chanmodes.c,v 5.18 2003/03/28 15:24:21 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "h.h"
#include "hook.h"
#include "msg.h"
#include "s_conf.h"
#include "chanmode.h"
#include "cmodetab.h"

static char buf[BUFSIZE];
static char modebuf[REALMODEBUFLEN], parabuf[REALMODEBUFLEN];
static char modebuf2[REALMODEBUFLEN], parabuf2[REALMODEBUFLEN];

static int hookid_send_modes = 0;

void init_chanmodes(void *unused)
{
    hookid_send_modes = hook_add_event("send mode list");
}

int add_id(aClient *cptr, aChannel *chptr, char *id, int maxnums, dlink_list * list)
{
    int cnt = 0;
    dlink_node *ptr;
    aNUH *nuhptr;

    for (ptr = list->head; ptr; ptr = ptr->next) {
	nuhptr = ptr->data;
	if (MyClient(cptr) && (++cnt >= maxnums)) {
	    send_me_numeric(cptr, ERR_BANLISTFULL, chptr, id);
	    return 0;
	} else if (nuhptr && !match(nuhptr->nuhstr, id))
	    return 0;
    }

    nuhptr = (aNUH *) MyMalloc(sizeof(aNUH));
    nuhptr->nuhstr = (char *) MyMalloc(strlen(id) + 1);
    DupString(nuhptr->nuhstr, id);

    if (IsPerson(cptr) && !IsChanHideOps(chptr)) {
	nuhptr->who = (char *) MyMalloc(strlen(cptr->name) + strlen(cptr->user->username) +
#ifdef USE_FAKEHOST
					strlen(IsFake(cptr) ? cptr->user->fakehost :
					       cptr->user->host) + 3);
#else
					strlen(cptr->user->host) + 3);
#endif

	(void) ircsprintf(nuhptr->who, "%C", cptr);
    } else if (IsChanHideOps(chptr)) {
	nuhptr->who = (char *) MyMalloc(strlen(chptr->chname) + 1);
	(void) strcpy(nuhptr->who, chptr->chname);
    } else {
	nuhptr->who = (char *) MyMalloc(strlen(cptr->name) + 1);
	(void) strcpy(nuhptr->who, cptr->name);
    }

    if (id[0] == '*' && id[1] == '!') {
	if (id[2] == '*' && id[3] == '@')
	    nuhptr->type = MTYP_HOST;
	else
	    nuhptr->type = MTYP_USERHOST;
    } else if (id[0] == '#')
	nuhptr->type = MTYP_CHANNEL;
    else
	nuhptr->type = MTYP_FULL;

    nuhptr->when = timeofday;

    ptr = make_dlink_node();
    dlinkAdd(nuhptr, ptr, list);

    return 1;
}

int del_id(char *id, dlink_list * list)
{
    aNUH *nuhptr;
    dlink_node *ptr, *next_ptr;

    if (!id)
	return -1;

    for (ptr = list->head; ptr; ptr = next_ptr) {
	next_ptr = ptr->next;
	nuhptr = ptr->data;
	if (!nuhptr)
	    continue;
	if (irc_strcmp(id, nuhptr->nuhstr) == 0) {
	    MyFree(nuhptr->nuhstr);
	    MyFree(nuhptr->who);
	    MyFree(nuhptr);
	    dlinkDelete(ptr, list);
	    free_dlink_node(ptr);
	    break;
	}
    }
    return 1;
}

void remove_matching_nuhs(aChannel *chptr, aClient *cptr, aClient *from,
			  dlink_list * list, char letter)
{
    aNUH *tmpA;
    dlink_node *ptr, *next_ptr;
    char targhost[NICKLEN + USERLEN + HOSTLEN + 6];
#ifdef USE_FAKEHOST
    char fakeuhost[NICKLEN + USERLEN + HOSTLEN + 6];
#endif
    char targip[NICKLEN + USERLEN + HOSTLEN + 6];
    char *m;
    int count = 0, send = 0;

    if (!IsPerson(cptr))
	return;

    strcpy(targhost, make_nick_user_host(cptr->name, cptr->user->username, cptr->user->host));
#ifdef USE_FAKEHOST
    strcpy(fakeuhost, make_nick_user_host(cptr->name, cptr->user->username, cptr->user->fakehost));
#endif
    strcpy(targip, make_nick_user_host(cptr->name, cptr->user->username, cptr->hostip));

    m = modebuf;
    *m++ = '-';
    *m = '\0';
    *parabuf = '\0';

    for (ptr = list->head; ptr; ptr = next_ptr) {
	next_ptr = ptr->next;
	tmpA = ptr->data;
	if (!tmpA)
	    continue;
	if (!tmpA->nuhstr)
	    continue;
	if ((match(tmpA->nuhstr, targhost) == 0)
	    || (match(tmpA->nuhstr, targip) == 0)
#ifdef USE_FAKEHOST
	    || (match(tmpA->nuhstr, fakeuhost) == 0)
#endif
	    ) {
	    if (strlen(parabuf) + strlen(tmpA->nuhstr) + 10 < (size_t) MODEBUFLEN) {
		if (*parabuf)
		    strcat(parabuf, " ");
		strcat(parabuf, tmpA->nuhstr);
		count++;
		*m++ = letter;
		*m = '\0';
	    } else if (*parabuf)
		send = 1;
	    if (count == MAXMODEPARAMS)
		send = 1;
	    if (send) {
		sendto_channel_butserv(chptr, from, TOK1_MODE, 0, "%s %s", modebuf, parabuf);
		sendto_flag_serv_butone(from, CAPAB_TMODE, 0, from, TOK1_TMODE, "%H %T %s %s",
					chptr, chptr, modebuf, parabuf);
		sendto_flag_serv_butone(from, 0, CAPAB_TMODE, from, TOK1_MODE, "%H %s %s", chptr,
					modebuf, parabuf);
		send = 0;
		*parabuf = '\0';
		m = modebuf;
		*m++ = '-';
		if (count != MAXMODEPARAMS) {
		    strcpy(parabuf, tmpA->nuhstr);
		    *m++ = letter;
		    count = 1;
		} else
		    count = 0;
		*m = '\0';
	    }
	    MyFree(tmpA->nuhstr);
	    MyFree(tmpA->who);
	    MyFree(tmpA);
	    dlinkDelete(ptr, list);
	    free_dlink_node(ptr);
	}
    }

    if (*parabuf) {
	sendto_channel_butserv(chptr, from, TOK1_MODE, 0, "%s %s", modebuf, parabuf);
	sendto_flag_serv_butone(from, CAPAB_TMODE, 0, from, TOK1_TMODE, "%H %T %s %s", chptr, chptr,
				modebuf, parabuf);
	sendto_flag_serv_butone(from, 0, CAPAB_TMODE, from, TOK1_MODE, "%H %s %s", chptr, modebuf,
				parabuf);
    }

    return;
}

void remove_nuh_list(aClient *cptr, aChannel *chptr, dlink_list * list, char letter)
{
    aNUH *bp = NULL;
    dlink_node *ptr, *next_ptr;
    char *cp;
    int count = 0, send = 0;
    cp = modebuf;
    *cp++ = '-';
    *cp = '\0';
    *parabuf = '\0';

    for (ptr = list->head; ptr; ptr = next_ptr) {
	next_ptr = ptr->next;
	bp = ptr->data;
	if (!bp)
	    continue;
	if (strlen(parabuf) + strlen(bp->nuhstr) + 10 < (size_t) MODEBUFLEN) {
	    if (*parabuf)
		strcat(parabuf, " ");
	    strcat(parabuf, bp->nuhstr);
	    count++;
	    *cp++ = letter;
	    *cp = '\0';
	} else if (*parabuf)
	    send = 1;
	if (count == MAXMODEPARAMS)
	    send = 1;
	if (send) {
	    sendto_channel_butserv(chptr, &me, TOK1_MODE, 0, "%s %s", modebuf, parabuf);
	    send = 0;
	    *parabuf = '\0';
	    cp = modebuf;
	    *cp++ = '-';
	    if (count != MAXMODEPARAMS) {
		strcpy(parabuf, bp->nuhstr);
		*cp++ = letter;
		count = 1;
	    } else
		count = 0;
	    *cp = '\0';
	}
	MyFree(bp->nuhstr);
	MyFree(bp->who);
	MyFree(bp);
	dlinkDelete(ptr, list);
	free_dlink_node(ptr);
    }

    if (*parabuf) {
	sendto_channel_butserv(chptr, &me, TOK1_MODE, 0, "%s %s", modebuf, parabuf);
    }

}

void send_nuh_list(aClient *cptr, aChannel *chptr, dlink_list * list,
		   char letter, char *mbuf, char *pbuf)
{
    char *cp;
    dlink_node *ptr;
    aNUH *bp;
    int count = 0, send = 0;
    cp = mbuf + strlen(mbuf);
    if (*pbuf)			/* mode +l or +k xx */
	count = 1;

    for (ptr = list->head; ptr; ptr = ptr->next) {
	bp = ptr->data;
	if (!bp)
	    continue;
	if (strlen(pbuf) + strlen(bp->nuhstr) + 10 < (size_t) MODEBUFLEN) {
	    if (*pbuf)
		strcat(pbuf, " ");
	    strcat(pbuf, bp->nuhstr);
	    count++;
	    *cp++ = letter;
	    *cp = '\0';
	} else if (*pbuf)
	    send = 1;
	if (count == MAXMODEPARAMS)
	    send = 1;
	if (send) {
	    if (IsTmode(cptr))
		sendto_one_server(cptr, &me, TOK1_TMODE, "%H %T %s %s", chptr, chptr, mbuf, pbuf);
	    else
		sendto_one_server(cptr, &me, TOK1_MODE, "%H %s %s", chptr, mbuf, pbuf);
	    send = 0;
	    *pbuf = '\0';
	    cp = mbuf;
	    *cp++ = '+';
	    if (count != MAXMODEPARAMS) {
		strcpy(pbuf, bp->nuhstr);
		*cp++ = letter;
		count = 1;
	    } else
		count = 0;
	    *cp = '\0';
	}
    }
    if (mbuf[1] || *pbuf) {
	if (IsTmode(cptr))
	    sendto_one_server(cptr, &me, TOK1_TMODE, "%H %T %s %s", chptr, chptr, mbuf, pbuf);
	else
	    sendto_one_server(cptr, &me, TOK1_MODE, "%H %s %s", chptr, mbuf, pbuf);
    }

}

aNUH *is_nuhed(aClient *cptr, dlink_list * list)
{
    char s[NICKLEN + USERLEN + HOSTLEN + 6];

#ifdef USE_FAKEHOST
    char fakeuhost[NICKLEN + USERLEN + HOSTLEN + 6];
#endif
    char *s2;
    dlink_node *ptr;
    aNUH *tmp;

    if (!IsPerson(cptr))
	return NULL;

    strcpy(s, make_nick_user_host(cptr->name, cptr->user->username, cptr->user->host));
    s2 = make_nick_user_host(cptr->name, cptr->user->username, cptr->hostip);
#ifdef USE_FAKEHOST
    strcpy(fakeuhost, make_nick_user_host(cptr->name, cptr->user->username, cptr->user->fakehost));
#endif
    for (ptr = list->head; ptr; ptr = ptr->next) {
	tmp = ptr->data;
	if (!tmp)
	    continue;
	if (!tmp->nuhstr)
	    continue;
	if ((match(tmp->nuhstr, s) == 0) || (match(tmp->nuhstr, s2) == 0)
#ifdef USE_FAKEHOST
	    || (match(tmp->nuhstr, fakeuhost) == 0)
#endif
	    )
	    return (tmp);
    }
    return NULL;
}

/* This function checks, if the attempted new /nick is denied
 * by a nuhptr
 * -TimeMr14C
 */

aNUH *nick_is_nuhed(aChannel *chptr, char *nick, aClient *cptr, dlink_list * list)
{
    dlink_node *ptr;
    aNUH *tmp = NULL;
    char *s, s2[NICKLEN + USERLEN + HOSTLEN + 6];
#ifdef USE_FAKEHOST
    char fakeuhost[NICKLEN + USERLEN + HOSTLEN + 6];
#endif
    if (!IsPerson(cptr))
	return NULL;
    strcpy(s2, make_nick_user_host(nick, cptr->user->username, cptr->user->host));
#ifdef USE_FAKEHOST
    strcpy(fakeuhost, make_nick_user_host(nick, cptr->user->username, cptr->user->fakehost));
#endif
    s = make_nick_user_host(nick, cptr->user->username, cptr->hostip);

    for (ptr = list->head; ptr; ptr = ptr->next) {
	tmp = ptr->data;
	if (!tmp)
	    continue;
	if (!tmp->nuhstr)
	    continue;
	if (tmp->type == MTYP_FULL &&	/* only check applicable bans */
	    ((match(tmp->nuhstr, s2) == 0) ||	/* check host before IP */
	     (match(tmp->nuhstr, s) == 0)
#ifdef USE_FAKEHOST
	     || (match(tmp->nuhstr, fakeuhost) == 0)
#endif
	    ))
	    return tmp;
    }
    return NULL;
}

/*
 * write the "simple" list of channel modes for channel chptr onto
 * buffer mbuf with the parameters in pbuf.
 */
void channel_modes(aClient *cptr, char *mbuf, char *pbuf, aChannel *chptr)
{
    *mbuf++ = '+';
    if (chptr->mode.mode & MODE_SECRET)
	*mbuf++ = 's';
    if (chptr->mode.mode & MODE_PRIVATE)
	*mbuf++ = 'p';
    if (chptr->mode.mode & MODE_MODERATED)
	*mbuf++ = 'm';
    if (chptr->mode.mode & MODE_TOPICLIMIT)
	*mbuf++ = 't';
    if (chptr->mode.mode & MODE_INVITEONLY)
	*mbuf++ = 'i';
    if (chptr->mode.mode & MODE_NOPRIVMSGS)
	*mbuf++ = 'n';
    if (chptr->mode.mode & MODE_REGISTERED)
	*mbuf++ = 'r';
    if (chptr->mode.mode & MODE_REGONLY)
	*mbuf++ = 'R';
    if (chptr->mode.mode & MODE_NOCOLOR)
	*mbuf++ = 'c';
    if (chptr->mode.mode & MODE_OPERONLY)
	*mbuf++ = 'O';
    if (chptr->mode.mode & MODE_ANONYMOUS)
	*mbuf++ = 'q';
    if (chptr->mode.mode & MODE_HIDEOPS)
	*mbuf++ = 'x';
    if (chptr->mode.mode & MODE_EXTOPIC)
	*mbuf++ = 'T';
    if (chptr->mode.mode & MODE_NONONRES)
	*mbuf++ = 'N';
    if (chptr->mode.mode & MODE_LIMIT) {
    char tmp[9];
    int len;
	*mbuf++ = 'l';
	if (IsMember(cptr, chptr) || IsPrivileged(cptr)) {
	    len = ircsprintf(tmp, " %d", chptr->mode.limit);
	    strncat(pbuf, tmp, len);
	}
    }
    if (chptr->mode.mode & MODE_KEY) {
    char k[KEYLEN + 1];
    int len;
	*mbuf++ = 'k';
	if (IsMember(cptr, chptr) || IsPrivileged(cptr)) {
	    len = ircsprintf(k, " %s", chptr->mode.key);
	    strncat(pbuf, k, len);
	}
    }
    if (chptr->mode.mode & MODE_FLOOD) {
    char t[20];
    int len;
	*mbuf++ = 'f';
	if (IsMember(cptr, chptr) || IsPrivileged(cptr)) {
	    len = ircsprintf(t, " %d:%d", chptr->mode.lines, chptr->mode.intime);
	    strncat(pbuf, t, len);
	}
    }
    // *pbuf++ = '\0';
    *mbuf++ = '\0';
    return;
}

/*
 * send "cptr" a full list of the modes for channel chptr. 
 */
void send_channel_modes(aClient *cptr, aChannel *chptr)
{
    dlink_node *ptr;
    aClient *acptr;
    int n = 0;
    int f;
    int len;
    char *t;
    struct hook_data thisdata;
    struct ChanMember *cm;

    if (*chptr->chname != '#')
	return;

    *modebuf = *parabuf = '\0';
    channel_modes(cptr, modebuf, parabuf, chptr);
    len = ircsprintf(buf, "%T %s %s%s :", chptr, chptr->chname, modebuf, parabuf);
    t = buf + len;

    for (ptr = chptr->members.head; ptr; ptr = ptr->next) {
        cm = ptr->data;
	if (!cm)
	    continue;
        acptr = cm->client_p;
        if (acptr == NULL)
            continue;  
        if (IsAnon(acptr))
            continue;
        f = cm->flags;
        if (f & CHFL_OWNER)
            *t++ = '.';
        if (f & CHFL_PROTECT)
            *t++ = '~';
        if (f & CHFL_HALFOP)
            *t++ = '%';
        if (f & CHFL_CHANOP)
            *t++ = '@';
        if (f & CHFL_VOICE)
            *t++ = '+';
        f = 0;
	strcpy(t, acptr->name);
	t += strlen(t);
	*t++ = ' ';
	n++;
	if (t - buf > BUFSIZE - 80) {
	    *t++ = '\0';
	    if (t[-1] == ' ')
		t[-1] = '\0';
	    sendto_one_server(cptr, &me, TOK1_SJOIN, "%s", buf);
	    len = ircsprintf(buf, "%T %s 0 :", chptr, chptr->chname);
	    t = buf + len;
	    n = 0;
	}
    }

    if (n) {
	*t++ = '\0';
	if (t[-1] == ' ')
	    t[-1] = '\0';
	sendto_one_server(cptr, &me, TOK1_SJOIN, "%s", buf);
    }

    thisdata.client_p = cptr;
    thisdata.channel = chptr;

    hook_call_event(hookid_send_modes, &thisdata);

}

int
set_mode(aClient *cptr, aClient *sptr, aChannel *chptr,
	 int parc, char *parv[], char *mbuf, char *pbuf)
{
    struct ChanMode *modetab_item;

    int adl = CMODE_LIST;

    char *modes = parv[0];	/* user's idea of mode changes */
    int args;			/* counter for what argument we're on */
    int maxmodes = ((IsServer(sptr) || IsULine(sptr)) ? 512 : MAXMODEPARAMS);
    int nmodes = 0;		/* how many modes we've set so far */
    int pidx = 0;		/* index into pbuf */
    int mbix = 0;
    int s;

    args = 1;

    if (parc < 1)
	return 0;

    mbuf[mbix++] = '+';

    mbuf[mbix] = '\0';
    pbuf[pidx] = '\0';

    /*
     * go through once to clean the user's mode string so we can
     * * have a simple parser run through it...
     */

    while (*modes && (nmodes < maxmodes)) {
	s = (u_char) *modes;
	if (*modes == '+') {
	    if (mbuf[(mbix - 1)] == '-') {
		mbuf[(mbix - 1)] = '+';	/* change it around now */
	    } else if (adl == CMODE_DEL)
		mbuf[mbix++] = '+';
	    adl = CMODE_ADD;

	} else if (*modes == '-') {
	    if (mbuf[(mbix - 1)] == '+') {
		mbuf[(mbix - 1)] = '-';	/* change it around now */
	    } else if (adl == CMODE_ADD)
		mbuf[mbix++] = '-';
	    adl = CMODE_DEL;

	} else if ((s > 31) && (s < 128)) {
	    modetab_item = &modetab[(u_char) *modes];
	    if (!(modetab_item->in_use)) {
		if (MyClient(sptr))
		    send_me_numeric(sptr, ERR_UNKNOWNMODE, *modes);
	    } else {
		if ((parc == 1) && modetab_item->listable)
		    adl = CMODE_LIST;
		if (IsService(sptr)) {
		    nmodes = (*modetab_item->cmode_handler_service) (adl, chptr,
								     nmodes,
								     &args, &pidx,
								     &mbix, mbuf,
								     pbuf, cptr, sptr, parc, parv);
		} else if (IsULine(sptr)) {
		    nmodes = (*modetab_item->cmode_handler_ulined) (adl, chptr,
								    nmodes,
								    &args, &pidx,
								    &mbix, mbuf,
								    pbuf, cptr, sptr, parc, parv);
		} else if (IsServer(sptr)) {
		    nmodes = (*modetab_item->cmode_handler_server) (adl, chptr,
								    nmodes,
								    &args, &pidx,
								    &mbix, mbuf,
								    pbuf, cptr, sptr, parc, parv);
		} else if (IsAnOper(sptr)) {
		    nmodes =
			(*modetab_item->cmode_handler_oper) (adl, chptr, nmodes,
							     &args, &pidx, &mbix,
							     mbuf, pbuf, cptr, sptr, parc, parv);
		} else {
		    nmodes =
			(*modetab_item->cmode_handler_user) (adl, chptr, nmodes,
							     &args, &pidx, &mbix,
							     mbuf, pbuf, cptr, sptr, parc, parv);
		}
	    }
	} else {
	    if (MyClient(sptr))
		send_me_numeric(sptr, ERR_UNKNOWNMODE, *modes);
	}
	modes++;
    }

    pbuf[pidx] = '\0';

    if (mbuf[(mbix - 1)] == '+' || mbuf[(mbix - 1)] == '-')
	mbuf[(mbix - 1)] = '\0';
    else
	mbuf[mbix] = '\0';

    return nmodes;
}

void send_mode_burst(aClient *cptr, aChannel *chptr, char change)
{
    char *c;
    int count = 0, send = 0;
    aClient *acptr;
    dlink_node *ptr;
    int f;
    struct ChanMember *cm;

    modebuf2[0] = '\0';
    parabuf2[0] = '\0';
    c = modebuf2;

    for (ptr = chptr->members.head; ptr; ptr = ptr->next) {
        cm = ptr->data;
        acptr = cm->client_p;
        f = cm->flags;
        if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) {
            if (f & CHFL_OWNER) {  
                *c++ = 'u';
                if (*parabuf2)
                    strcat(parabuf2, " "); 
                strcat(parabuf2, acptr->name);
                count++;
            }
        }
        if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) {
            if (f & CHFL_PROTECT) {
                *c++ = 'a';
                if (*parabuf2)
                    strcat(parabuf2, " ");
                strcat(parabuf2, acptr->name);
                count++;
            }
        }
        if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) {
            if (f & CHFL_CHANOP) {
                *c++ = 'o';
                if (*parabuf2)
                    strcat(parabuf2, " ");
                strcat(parabuf2, acptr->name);
                count++;
            }
        }
        if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) { 
            if (f & CHFL_HALFOP) {
                *c++ = 'h';
                if (*parabuf2)
                    strcat(parabuf2, " ");
                strcat(parabuf2, acptr->name);
                count++;
            }
        }
        if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) {
            if (f & CHFL_VOICE) {
                *c++ = 'v';
                if (*parabuf2)
                    strcat(parabuf2, " ");
                strcat(parabuf2, acptr->name);
                count++;
            }
        }
	if (strlen(parabuf2) + strlen(acptr->name) + 10 < (size_t) MODEBUFLEN) {
	    *c = '\0';
	} else if (*parabuf2)
	    send = 1;
	if (count == MAXMODEPARAMS)
	    send = 1;
        if (send) {
            sendto_one(cptr, ":%H %s %H %c%s %s", chptr, MSG_MODE,
                       chptr, change, modebuf2, parabuf2);
            send = 0;
            *parabuf2 = '\0';
            c = modebuf2;
            if (count != MAXMODEPARAMS) {
                if (f & CHFL_OWNER) {
                    *c++ = 'u';
                    strcat(parabuf2, acptr->name);
                    strcat(parabuf2, " ");
                    count++;
                }
                if (f & CHFL_PROTECT) {
                    *c++ = 'a';
                    strcat(parabuf2, acptr->name);
                    strcat(parabuf2, " ");
                    count++;
                }
                if (f & CHFL_CHANOP) {
                    *c++ = 'o';
                    strcat(parabuf2, acptr->name);
                    strcat(parabuf2, " ");
                    count++;
                }
                if (f & CHFL_HALFOP) {
                    *c++ = 'h';    
                    strcat(parabuf2, acptr->name);
                    strcat(parabuf2, " ");
                    count++;
                }
                if (f & CHFL_VOICE) {
                    *c++ = 'v';
                    strcat(parabuf2, acptr->name);
                    strcat(parabuf2, " ");
                    count++;
                }
            } else
                count = 0;
            *c = '\0';
        }
    }
    if (*parabuf2) {
	sendto_one(cptr, ":%H %s %H %c%s %s", chptr, MSG_MODE, chptr, change, modebuf2, parabuf2);
    }
    *parabuf2 = '\0';
    *modebuf2 = '\0';
}
