/************************************************************************
 *   IRC - Internet Relay Chat, src/ircd.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: ircd.c,v 5.22 2003/05/15 14:52:43 tr-ircd Exp $ 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/stat.h>
#include <limits.h>
#include <sys/file.h>
#include <pwd.h>
#include <signal.h>
#include <sys/resource.h>
#include <sys/ioctl.h>
#include <sys/wait.h>

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "channel.h"
#include "msg.h"
#include "h.h"
#include "optparse.h"
#include "dh.h"
#include "hook.h"
#include "tools.h"
#include "s_auth.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "event.h"
#include "modules.h"
#include "language.h"
#include "throttle.h"
#include "comply.h"
#include "packet.h"

#ifndef TRUE
#define TRUE 1
#endif

static unsigned long initialVMTop = 0;	/* top of virtual memory at init */

aClient me;                     /* That's me */
aClient *GlobalClientList;      /* Pointer to beginning of Client list */

aConfList KList1 = { 0, NULL };	/* ordered */
aConfList KList2 = { 0, NULL };	/* ordered, reversed */
aConfList KList3 = { 0, NULL };	/* what we can't sort */

aConfList ZList1 = { 0, NULL };	/* ordered */

struct Counter Count;

aConfItem *GlobalConfItemList;	/* global confitem beginning */
aConfItem *temporary_excludes = (aConfItem *) NULL;
aConfItem *temporary_klines = (aConfItem *) NULL;
aConfItem *szlines = (aConfItem *) NULL;

int dorehash = 0;
int rehashed = 1;

int callbacks_called = 0;	/* A measure of server load... */

time_t NOW;

void server_reboot();
void restart(char *);
static void io_loop();

char **myargv;

char REPORT_DO_DNS[256];
char REPORT_FIN_DNS[256];
char REPORT_FIN_DNSC[256]; 
char REPORT_FAIL_DNS[256];
char REPORT_DO_ID[256];
char REPORT_FIN_ID[256];
char REPORT_FAIL_ID[256];

dlink_list lclient_list;
dlink_list serv_list;
dlink_list global_serv_list;
dlink_list unknown_list;
dlink_list locoper_list;
dlink_list exit_list;

dlink_list u_conf;
dlink_list h_conf;

struct server_options ServerOpts;
struct server_info ServerInfo;
struct services_conf ServicesConf;
struct admin_info AdminInfo;
struct channel_conf ChannelConf;
struct server_hide ServerHide;
struct general_options GeneralOpts;
struct log_system LogSys;

/*
 * get_vm_top - get the operating systems notion of the resident set size
 */
static unsigned long get_vm_top(void)
{
    /*
     * NOTE: sbrk is not part of the ANSI C library or the POSIX.1 standard
     * however it seems that everyone defines it. Calling sbrk with a 0
     * argument will return a pointer to the top of the process virtual
     * memory without changing the process size, so this call should be
     * reasonably safe (sbrk returns the new value for the top of memory).
     * This code relies on the notion that the address returned will be an 
     * offset from 0 (NULL), so the result of sbrk is cast to a size_t and 
     * returned. We really shouldn't be using it here but...
     */
    void *vptr = sbrk(0);
    return (unsigned long) vptr;
}

void recheck_clock(void *unused)
{
    char to_send[200];
    struct timeval newtime;
    newtime.tv_sec = 0;
    newtime.tv_usec = 0;

    if (gettimeofday(&newtime, NULL) == -1) {
	sendto_ops("Clock Failure (%d), TS can be corrupted", errno);
	restart("Clock Failure");
    }

    if (newtime.tv_sec < timeofday) {
	ircsprintf(to_send,
		   "System clock is running backwards - (%lu < %lu)", 
		   newtime.tv_sec, timeofday);
	set_back_events(timeofday - newtime.tv_sec);
    }
    timeofday = newtime.tv_sec;
}

static int irc_sleep(unsigned long useconds)
{
#ifdef HAVE_NANOSLEEP
    struct timespec t;
    t.tv_sec = useconds / (unsigned long) 1000000;
    t.tv_nsec = (useconds % (unsigned long) 1000000) * 1000;
    return nanosleep(&t, (struct timespec *) NULL);
#else
    struct timeval t;
    t.tv_sec = 0;
    t.tv_usec = useconds;
    return select(0, NULL, NULL, NULL, &t);
#endif
}

static void save_stats(void *unused)
{
    FILE *fp;

    fp = fopen(IRCD_PREFIX_VAR "/maxclients", "w");
    if (fp != NULL) {
	fprintf(fp, "%d %d %li %li %li %ld %ld %ld %ld", Count.max_loc,
		Count.max_tot, Count.weekly, Count.monthly,
		Count.yearly, Count.start, Count.week, Count.month, Count.year);
	fclose(fp);
	sendto_lev(DEBUG_LEV, "Saved maxclient statistics");
    }
}
void write_pidfile()
{
int fd;
char buff[20];

    if ((fd = open(IRCD_PIDFILE, O_CREAT | O_WRONLY, 0600)) >= 0) {
	(void) ircsprintf(buff, "%5d\n", (int) getpid());
	if (write(fd, buff, strlen(buff)) == -1)
	    logevent_call(LogSys.pid, "writing", IRCD_PIDFILE);
	(void) close(fd);
	return;
    } else
	logevent_call(LogSys.pid, "opening", IRCD_PIDFILE);
}

void init_sys(void)
{
    FILE *mcsfp;

#ifdef RLIMIT_FD_MAX
    struct rlimit limit;

    if (!getrlimit(RLIMIT_FD_MAX, &limit)) {

	if (limit.rlim_max < MAXCONNECTIONS) {
	    fprintf(stderr, "ircd fd table too big\n");
	    fprintf(stderr, "Hard Limit: %ld IRC max: %d\n", (long) limit.rlim_max, MAXCONNECTIONS);
	    fprintf(stderr, "Fix MAXCONNECTIONS\n");
	    exit(-1);
	}

	limit.rlim_cur = limit.rlim_max;	/* make soft limit the max */
	if (setrlimit(RLIMIT_FD_MAX, &limit) == -1) {
	    fprintf(stderr, "error setting max fd's to %ld\n", (long) limit.rlim_cur);
	    exit(-1);
	}
    }
#endif /* RLIMIT_FD_MAX */

    mcsfp = fopen(IRCD_PREFIX_VAR "/maxclients", "r");
    if (mcsfp != NULL) {
        fscanf(mcsfp, "%d %d %li %li %li %ld %ld %ld %ld", &Count.max_loc,
               &Count.max_tot, &Count.weekly, &Count.monthly,
               &Count.yearly, &Count.start, &Count.week, &Count.month,          
               &Count.year);
        fclose(mcsfp);
    }                  

}

/*
 * setup_corefile
 *
 * inputs       - nothing
 * output       - nothing
 * side effects - setups corefile to system limits.
 * -kre
 */
static void setup_corefile(void)
{
#if defined(HAVE_SYS_RESOURCE_H)
    struct rlimit rlim;		/* resource limits */

    /* Set corefilesize to maximum */
    if (!getrlimit(RLIMIT_CORE, &rlim)) {
	rlim.rlim_cur = rlim.rlim_max;
	setrlimit(RLIMIT_CORE, &rlim);
    }
#endif
}

static void initialize_generalopts(void)
{
    GeneralOpts.debuglevel = 0;
    GeneralOpts.foreground = 0;
    GeneralOpts.maxclients = D_MAX_CLIENTS;
    GeneralOpts.superiordifference = 0;
    GeneralOpts.protocol_in_use = 0;
    GeneralOpts.spam_num = 0;
    GeneralOpts.spam_time = 0;
    GeneralOpts.still_on_split = 0;

    GeneralOpts.dpath = IRCD_PREFIX;
    GeneralOpts.configfile = CONFIGFILE;
}


/*
 * cleanup_zombies
 * inputs       - nothing
 * output       - nothing
 * side effects - Reaps zombies periodically
 * -AndroSyn
 */
static void cleanup_zombies(void *unused)
{
    int status;
    waitpid(-1, &status, WNOHANG);
}

static int make_daemon(void)
{
    int pid;
    printf("TR-IRCD5 is now becoming a daemon\n");
    if (GeneralOpts.foreground == 0) {
    	if ((pid = fork()) < 0) {
	    perror("fork");
	    exit(-1);
    	} else if (pid > 0) {
	    exit(EXIT_SUCCESS);
        }
    }
    setsid();
    return 0;
}

int main(int argc, char *argv[])
{
    build_version();

    Count.server = 1;		/* it's just me */
    Count.start = NOW;
    Count.day = NOW;
    Count.week = NOW;
    Count.month = NOW;
    Count.year = NOW;

    /*
     * this code by mika@cs.caltech.edu 
     * it is intended to keep the ircd from being swapped out. BSD
     * swapping criteria do not match the requirements of ircd
     */

    myargv = argv;
    (void) umask(077);		/* better safe than sorry --SRB */

    memset(&me, 0, sizeof(me));

    memset(&unknown_list, 0, sizeof(unknown_list));
    memset(&lclient_list, 0, sizeof(lclient_list));
    memset(&serv_list, 0, sizeof(serv_list));
    memset(&global_serv_list, 0, sizeof(global_serv_list));
    memset(&locoper_list, 0, sizeof(locoper_list));
    memset(&exit_list, 0, sizeof(exit_list));

    unknown_list.head = unknown_list.tail = NULL;
    lclient_list.head = lclient_list.tail = NULL;
    serv_list.head = serv_list.tail = NULL;
    global_serv_list.head = global_serv_list.tail = NULL;
    locoper_list.head = locoper_list.tail = NULL;
    exit_list.head = exit_list.tail = NULL;

    memset(&u_conf, 0, sizeof(u_conf));
    memset(&h_conf, 0, sizeof(h_conf));

    u_conf.head = u_conf.tail = NULL;
    h_conf.head = h_conf.tail = NULL;

    memset((void *) &ServerInfo, 0, sizeof(ServerInfo));
    memset((void *) &AdminInfo, 0, sizeof(AdminInfo));
    memset((void *) &ServerOpts, 0, sizeof(ServerOpts));
    memset((void *) &ChannelConf, 0, sizeof(ChannelConf));
    memset((void *) &ServicesConf, 0, sizeof(ServicesConf));
    memset((void *) &GeneralOpts, 0, sizeof(GeneralOpts));
    memset((void *) &LogSys, 0, sizeof(LogSys));

    GlobalClientList = &me;	/* Pointer to beginning of Client list */

    setup_corefile();

    /*
     * set initialVMTop before we allocate any memory
     */
    initialVMTop = get_vm_top();

    setup_signals();

    initialize_generalopts();

    init_optparse(NULL);

    process_commandline(argc, argv);

    if (chdir(GeneralOpts.dpath)) {
	perror("chdir");
	fprintf(stderr,
		"Please ensure that your configuration directory exists and is accessable.\n");
	exit(-1);
    }

#ifdef HAVE_ENCRYPTION_ON
    if (dh_init() == -1)
	return 0;
#endif

    clear_client_hash_table();
    clear_channel_hash_table();
    clear_scache_hash_table();	/* server cache name table */

    recheck_clock(NULL);

    init_sys();

    make_daemon();

    fdlist_init();

    initBlockHeap();
    init_dlink_nodes();
    linebuf_init();

    init_logsystem(NULL);       /* Log System initialize */
    init_logevents(NULL);       /* Log Events initialize */
    init_log_elems(NULL);
    init_log_files(NULL);       /* Log Files opening */

    initlists();
    initclass();
    initwhowas();
    initstats();

    init_netio();		/* This needs to be setup early ! -- adrian */
    init_resolver();		/* Needs to be setup before the io loop */
    init_hooks();
    init_dopacket(NULL);

    clear_hash_parse();
    clear_hash_lang();
    init_channel(NULL);
    init_chanmodes(NULL);

    init_auth();		/* Initialise the auth code */
    eventInit();		/* Initialise event manager */

    init_modules();		/* Initialise modules */
    init_loader(NULL);

    init_message_file(MF_MOTD, MOTD, &(GeneralOpts.motd));
    init_message_file(MF_SHORTMOTD, SHORTMOTD, &(GeneralOpts.shortmotd));
    init_message_file(MF_LINKS, LINKSFILE, &(GeneralOpts.linksfile));
    init_message_file(MF_CONF, CONFIGFILE, &(GeneralOpts.conffile));

    read_message_file(&(GeneralOpts.motd));
    read_message_file(&(GeneralOpts.linksfile));
    read_message_file(&(GeneralOpts.shortmotd));
    read_message_file(&(GeneralOpts.conffile));

    init_user();
    init_server();

    NOW = irc_time(NULL);

    read_conf_files(1);

    load_all_modules(LM_MODULES | LM_LANG, 0);

    if (!init_protocol(ServerInfo.servermode)) {
	logevent_call(LogSys.generalerror, "Failed to initialize protocol module");
	logevent_call(LogSys.generalerror, "Server is exiting...");
	exit(-1);
    } 

    GeneralOpts.protocol_in_use = protocol_rehash();

    load_all_modules(LM_CHANMODES|LM_CONTRIB, 0);

    load_dependencies();

    GeneralOpts.spam_num = ServerOpts.max_join_leave_count;
    GeneralOpts.spam_time = ServerOpts.min_join_leave_time;

    throttle_init();		/* init the throttle system -wd */

    check_class();
    write_pidfile();

    iprintf_init(NULL);

#ifdef USE_IAUTH

    /* Warning: This is an unfinished code, since there is no
     * consensus stating what iauth shall do. -TimeMr14C
     */

    iAuth.flags = 0;

    ConnectToIAuth();

    if (iAuth.socket == NOSOCK) {
	fprintf(stderr, "Unable to connect to IAuth server\n");
	exit(-1);
    }
#endif

    /* Config file validation is in read_conf_files
     * that way we do not need to check items here again */

    strlcpy_irc(me.name, ServerInfo.name, HOSTLEN);

    me.fd = -1;
    me.hopcount = 0;
    me.authfd = -1;
    me.confs.head = NULL;
    me.next = NULL;
    me.user = NULL;
    me.from = &me;
    me.servptr = &me;

    SetMe(&me);
    make_server(&me);

    make_user(&me);		/* -TimeMr14C */

    me.serv->up = me.name;
    me.lasttime = me.since = me.firsttime = NOW;
    DupString(me.user->server, me.name);
    strlcpy_irc(me.user->username, "local", sizeof(me.user->username));
    strcpy(me.user->host, me.name);

    add_to_client_hash_table(me.name, &me);
    add_server_to_list(&me);	/* add ourselves to global_serv_list */

    init_levs(&me);

    if (init_identity(ServerInfo.identity ? ServerInfo.identity : 0xFFFF))
	logevent_call(LogSys.generalerror, "Invalid identity specified.");

    logevent_call(LogSys.generalerror, "Server ready...");

    NOW = irc_time(NULL);

    eventAddIsh("Check connections", try_connections, NULL, 60);
    eventAddIsh("Check timeouts", comm_checktimeouts, NULL, 1);
    eventAddIsh("Check pings", check_pings, NULL, 10);
    eventAddIsh("Delete throttles", throttle_timer_zapped, NULL, THROTTLE_LENGTH);
    eventAddIsh("cleanup_zombies", cleanup_zombies, NULL, 30);
#ifdef HAVE_ZIPLINKS_ON
    eventAddIsh("Flush server connections", flush_connections, NULL, 1);
#endif
    eventAddIsh("Save stats", save_stats, NULL, ServerOpts.save_maxclient_stats_time);
    eventAddIsh("Garbage collection", block_garbage_collect, NULL, 30);
    eventAddIsh("remove exited clients", remove_exited_clients, NULL, 30);

    if (ServerHide.links_from_file)
	eventAddIsh("write_links_file", write_links_file, NULL, ServerHide.links_delay);

    io_loop();

    return 0;
}

static void io_loop()
{
int delay = 0, empty_cycles = 0, st = 0;
    while (1) {

	delay = eventNextTime();
	if (delay <= timeofday)
	    eventRun();
	NOW = timeofday;

	if (callbacks_called > 0)
	    empty_cycles = 0;

	else
	    empty_cycles++;

	/* Reset the callback counter... */
	callbacks_called = 0;
	st = (empty_cycles + 1) * 15000;
	if (st > 250000)
	    st = 250000;

	irc_sleep(st);

	comm_select(0);

	if (dorehash) {
	    (void) rehash(&me, &me, 1);
	    dorehash = 0;
	}

    }
}

