/************************************************************************
 *   IRC - Internet Relay Chat, src/memory.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: memory.c,v 5.7 2002/12/16 22:09:44 tr-ircd Exp $ 
 */

#define MALLOC_CHECK_ 	1

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "numeric.h"
#include "blalloc.h"
#include "s_conf.h"

void *MyMalloc(size_t x)
{
void *ret;

    ret = calloc(1, x);

    if (!ret) 
	outofmemory("MyMalloc");

    return ret;
}

void *MyRealloc(void *x, size_t y)
{
void *ret;

    ret = realloc(x, y);

    if (!ret)
        outofmemory("MyRealloc");

    return ret; 
}

/*
 * outofmemory()
 * 
 * input                - NONE output           - NONE side effects     -
 * simply try to report there is a problem I free all the memory in the
 * kline lists hoping to free enough memory so that a proper report can
 * be made. If I was already here (was_here) then I got called twice,
 * and more drastic measures are in order. I'll try to just abort() at
 * least. -Dianora
 */

void outofmemory(char *msg)
{
static int was_here = 0;

    if (was_here)
	abort();

    was_here = 1;
    clear_conf_list(&KList1);
    clear_conf_list(&KList2);
    clear_conf_list(&KList3);
    logevent_call(LogSys.serverkill, msg);
    logevent_call(LogSys.serverkill, "Out of memory");
    restart("Out of Memory");
}

/*
 * Attempt to free up some block memory
 *
 * list_garbage_collect
 * 
 * inputs               - NONE output           - NONE side effects     -
 * memory is possibly freed up
 */

void block_garbage_collect(void *unused)
{
    BlockHeapGarbageCollect(free_local_aClients);
    BlockHeapGarbageCollect(free_remote_aClients);
    BlockHeapGarbageCollect(free_channels);
    BlockHeapGarbageCollect(dnode_heap);
    BlockHeapGarbageCollect(hashent_freelist);
    BlockHeapGarbageCollect(throttle_freelist);
}
