#!/bin/sh
spitshell=cat
package="trircd-5"
md5sum=$1

echo "Extracting $package/src/version.c..."

creation=`date | \
awk '{if (NF == 6) \
{ print $1 " "  $2 " " $3 " "  $6 " at " $4 " " $5 } \
else \
{ print $1 " "  $2 " " $3 " " $7 " at " $4 " " $5 " " $6 }}'`

if test -r version.c.last
then
        generation=`sed -n 's/^char \*generation = \"\(.*\)\";/\1/p' < version.c.last`
        genesis=`sed -n 's/^char \*genesis = \"\(.*\)\";/\1/p' < version.c.last`
if test ! "$generation" ; then generation=0; fi
if test ! "$genesis" ; then genesis=$creation; fi
else
        generation=0
        genesis=$creation
fi

generation=`expr $generation + 1`

uname=`uname -a | sed 's/$/\",/;s/^/   "/'`
ulimit=`ulimit -a | sed 's/$/\",/;s/^/   "/'`

creation=`date | \
awk '{if (NF == 6) \
{ print $1 " "  $2 " " $3 " "  $6 " at " $4 " " $5 } \
else \
{ print $1 " "  $2 " " $3 " " $7 " at " $4 " " $5 " " $6 }}'`

$spitshell > version.c <<!SUB!THIS!
/*
 *   IRC - Internet Relay Chat, src/version.c
 *   Copyright (C) 1990 Chelsea Ashley Dyerman
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This file is generated by version.c.SH. Any changes made will go away.
 */

#include "struct.h"
#include "patchlevel.h"
#include "ircsprintf.h"

char *generation = "$generation";
char *creation = "$creation";
char version[128];

void build_version(void)
{
    char *s = PATCHES;

    ircsprintf(version, "%s(%s)-%.1d.%.1d(%.2d)%s#%s", BASENAME, BRANCH,
               MAJOR, MINOR, PATCH, (*s != 0 ? PATCHES : ""), generation);

}

char *infotext[] =
    {
	 "$package --",
	 "Based on the original code written by Jarkko Oikarinen",
	 "Copyright 1988, 1989, 1990, 1991 University of Oulu, Computing Center",
	 "",
	 "This program is free software; you can redistribute it and/or",
	 "modify it under the terms of the GNU General Public License as",
	 "published by the Free Software Foundation; either version 2, or",
	 "(at your option) any later version.",
	 "",
	 "Send bug reports to tr-ircd@users.sourceforge.net",
	 "",
         "TimeMr14C           Yusuf Iskenderoglu  Head Developer of TR-IRCD",
         "Talesin             Ekim Engin          Many source fixes",
	 "MySql               Grkan Yilmaz       Webmaster",
	 "",
	 "Special thanks to Andreas Haeberlen because of his help",
 	 "in perfecting many different code sections",
	 "Special thanks to Hakan Onur because of his ideas in perfecting",
	 "different code sections",  
	 "",
	 "This ircd software contains code from Bahamut, the DALnet ircd",
	 "and from hybrid, the EFnet ircd.",
	 "Vers special thanks go to the coders of these ircds.",
	 "",
	 "Thanks also goes to those persons not mentioned here who have added",
	 "their advice, opinions, and code to IRC.",
	 "Thanks also to those who provide the kind sys admins who let me and",
	 "others continue to develop IRC.",
	 "(c) By TR-IRCD Development Team 2002",
    "",
	 0,
};

char *uname_result[] = {
$uname
   0
};

char *ulimit_result[] = {
$ulimit
   0
};

char *srcchecksums[] = {
!SUB!THIS!

echo -n "Extracting checksums in src... "

SOURCE=" "
for i in `grep "^SOURCE" Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC1=`grep "^${i}" Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC1}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum $i 2>/dev/null`\"," >> version.c
done

echo "done"

STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "src/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *modulechecksums[] = {" >> version.c

echo -n "Extracting checksums in modules... "

SOURCE=" "
for i in `grep "^SOURCE" ../modules/Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC2=`grep "^${i}" ../modules/Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC2}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum ../modules/$i 2>/dev/null | sed -e 's/\.\.\/modules\///g'`\"," >> version.c
done

echo "done"

cd ../modules/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../src/

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "modules/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *serverchecksums[] = {" >> version.c

echo -n "Extracting checksums in server... "

SOURCE=" "
for i in `grep "^SOURCE" ../server/Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC3=`grep "^${i}" ../server/Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC3}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum ../server/$i 2>/dev/null | sed -e 's/\.\.\/server\///g'`\"," >> version.c
done

echo "done"

cd ../server/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../src/

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "server/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *cmodechecksums[] = {" >> version.c

echo -n "Extracting checksums in chanmodes... "

SOURCE=" "
for i in `grep "^SOURCE" ../chanmodes/Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC4=`grep "^${i}" ../chanmodes/Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC4}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum ../chanmodes/$i 2>/dev/null | sed -e 's/\.\.\/chanmodes\///g'`\"," >> version.c
done

echo "done" 

cd ../chanmodes/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../src/

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "chanmodes/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *logsyssums[] = {" >> version.c

echo -n "Extracting checksums in logsystem... "   

SOURCE=" "
for i in `grep "^SOURCE" logsystem/Makefile | awk -F= {'print $1'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC5=`grep "^${i}" logsystem/Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC5}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum logsystem/$i 2>/dev/null | sed -e 's/logsystem\///g'`\"," >> version.c
done

echo "done"

cd logsystem/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c                                                                               
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "src/logsystem/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *protocolsums[] = {" >> version.c

echo -n "Extracting checksums in protocol... "

SOURCE=" "
for i in `grep "^SOURCE" ../protocol/Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do
 SRC6=`grep "^${i}" ../protocol/Makefile | awk -F= {'print $2'}`
 SOURCE="${SOURCE} ${SRC6}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum ../protocol/$i 2>/dev/null | sed -e 's/\.\.\/protocol\///g'`\"," >> version.c
done

echo "done"

cd ../protocol/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../src/

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "protocol/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
echo "char *includechecksums[] = {" >> version.c

echo -n "Extracting checksums in includes... " 

SOURCE=" "
for i in `grep "^SOURCE" ../include/Makefile | awk -F= {'print $2'} | sed -e 's/[$,{,(,),}]//g'` ; do 
 SRC2=`grep "^${i}" ../include/Makefile | awk -F= {'print $2'}` 
 SOURCE="${SOURCE} ${SRC2}"
done
for i in ${SOURCE} ; do
   echo "       \"`$md5sum ../include/$i 2>/dev/null | sed -e 's/\.\.\/include\///g'`\"," >> version.c  
done

echo "done"

cd ../include/
STATUS=`$md5sum --check .md5sums 2>/dev/null`
iprev="dummy"
modded=""
cd ../src/

for i in $STATUS; do
   if test $i = "FAILED"; then
      echo "    \"$iprev FAILED\", " >> version.c;
      modded="$modded `echo $iprev | sed s/://`";
   fi;
   iprev=$i;
done

echo "  0" >> version.c
echo "};" >> version.c

if test "$modded" != ""; then
   echo "*** Modified files in section "include/" detected"
   for i in $modded; do
      echo "*** $i"
   done
fi

echo " " >> version.c
